#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.system_interfaces.setting_controller import SettingController


class WorkGeneralSettingAtt(SettingController):

    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     System.SettingController
                |                         WorkGeneralSettingAtt
                | 
                | Interface representing xxx.
                | 
                | Role: Components that implement DNBIAWorkGeneralSettingAtt are
                | ...
                | 
                | Do not use the DNBIAWorkGeneralSettingAtt interface for such and such
                | ClassReference, Class#MethodReference, #InternalMethod...
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.work_general_setting_att = com_object

    @property
    def annotation_activity(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property AnnotationActivity() As boolean
                | 
                |     Returns or sets the AnnotationActivity parameter.
                | 
                |     Ensure consistency with the C++ interface to which the work is delegated.

        :rtype: bool
        """

        return self.work_general_setting_att.AnnotationActivity

    @annotation_activity.setter
    def annotation_activity(self, value: bool):
        """
        :param bool value:
        """

        self.work_general_setting_att.AnnotationActivity = value

    @property
    def dnbwi_num_column_list(self) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property DNBWINumColumnList() As CATSafeArrayVariant
                | 
                |     Returns the DNBWINumColumnList parameter.
                | 
                |     Ensure consistency with the C++ interface to which the work is delegated.

        :rtype: tuple
        """

        return self.work_general_setting_att.DNBWINumColumnList

    @dnbwi_num_column_list.setter
    def dnbwi_num_column_list(self, value: tuple):
        """
        :param tuple value:
        """

        self.work_general_setting_att.DNBWINumColumnList = value

    @property
    def move_activity(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property MoveActivity() As boolean
                | 
                |     Returns or sets the MoveActivity parameter.
                | 
                |     Ensure consistency with the C++ interface to which the work is delegated.

        :rtype: bool
        """

        return self.work_general_setting_att.MoveActivity

    @move_activity.setter
    def move_activity(self, value: bool):
        """
        :param bool value:
        """

        self.work_general_setting_att.MoveActivity = value

    @property
    def style_sheet_path(self) -> str:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property StyleSheetPath() As CATBSTR
                | 
                |     Returns or sets the StyleSheetPath parameter.
                | 
                |     Ensure consistency with the C++ interface to which the work is delegated.

        :rtype: str
        """

        return self.work_general_setting_att.StyleSheetPath

    @style_sheet_path.setter
    def style_sheet_path(self, value: str):
        """
        :param str value:
        """

        self.work_general_setting_att.StyleSheetPath = value

    @property
    def text_activity(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property TextActivity() As boolean
                | 
                |     Returns or sets the TextActivity parameter.
                | 
                |     Ensure consistency with the C++ interface to which the work is delegated.

        :rtype: bool
        """

        return self.work_general_setting_att.TextActivity

    @text_activity.setter
    def text_activity(self, value: bool):
        """
        :param bool value:
        """

        self.work_general_setting_att.TextActivity = value

    @property
    def view_point_activity(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property ViewPointActivity() As boolean
                | 
                |     Returns or sets the ViewPointActivity parameter.
                | 
                |     Ensure consistency with the C++ interface to which the work is delegated.

        :rtype: bool
        """

        return self.work_general_setting_att.ViewPointActivity

    @view_point_activity.setter
    def view_point_activity(self, value: bool):
        """
        :param bool value:
        """

        self.work_general_setting_att.ViewPointActivity = value

    @property
    def visibility_activity(self) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property VisibilityActivity() As boolean
                | 
                |     Returns or sets the VisibilityActivity parameter.
                | 
                |     Ensure consistency with the C++ interface to which the work is delegated.

        :rtype: bool
        """

        return self.work_general_setting_att.VisibilityActivity

    @visibility_activity.setter
    def visibility_activity(self, value: bool):
        """
        :param bool value:
        """

        self.work_general_setting_att.VisibilityActivity = value

    def get_annotation_activity_info(self, io_admin_level: str, io_locked: str) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetAnnotationActivityInfo(CATBSTR ioAdminLevel,
                | CATBSTR ioLocked) As boolean
                | 
                |     Retrieves environment informations for the AnnotationActivity
                |     parameter.
                |     Role:Retrieves the state of the AnnotationActivity parameter in the current
                |     environment.
                | 
                |     Parameters:
                | 
                |         ioAdminLevel
                | 
                |             If the parameter is locked, AdminLevel gives the administration
                |             level that imposes the value of the parameter.
                |             If the parameter is not locked, AdminLevel gives the administration
                |             level that will give the value of the parameter after a reset.
                |             
                |         ioLocked
                |             Indicates if the parameter has been locked. 
                | 
                |     Returns:
                |         Indicates if the parameter has been explicitly modified or remain to
                |         the administrated value.

        :param str io_admin_level:
        :param str io_locked:
        :rtype: bool
        """
        return self.work_general_setting_att.GetAnnotationActivityInfo(io_admin_level, io_locked)

    def get_dnbwi_num_column_list_info(self, io_admin_level: str, io_locked: str) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetDNBWINumColumnListInfo(CATBSTR ioAdminLevel,
                | CATBSTR ioLocked) As boolean
                | 
                |     Retrieves environment informations for the DNBWINumColumnList
                |     parameter.
                |     Role:Retrieves the state of the DNBWINumColumnList parameter in the current
                |     environment.
                | 
                |     Parameters:
                | 
                |         ioAdminLevel
                | 
                |             If the parameter is locked, AdminLevel gives the administration
                |             level that imposes the value of the parameter.
                |             If the parameter is not locked, AdminLevel gives the administration
                |             level that will give the value of the parameter after a reset.
                |             
                |         ioLocked
                |             Indicates if the parameter has been locked. 
                | 
                |     Returns:
                |         Indicates if the parameter has been explicitly modified or remain to
                |         the administrated value.

        :param str io_admin_level:
        :param str io_locked:
        :rtype: bool
        """
        return self.work_general_setting_att.GetDNBWINumColumnListInfo(io_admin_level, io_locked)

    def get_move_activity_info(self, io_admin_level: str, io_locked: str) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetMoveActivityInfo(CATBSTR ioAdminLevel,
                | CATBSTR ioLocked) As boolean
                | 
                |     Retrieves environment informations for the MoveActivity
                |     parameter.
                |     Role:Retrieves the state of the MoveActivity parameter in the current
                |     environment.
                | 
                |     Parameters:
                | 
                |         ioAdminLevel
                | 
                |             If the parameter is locked, AdminLevel gives the administration
                |             level that imposes the value of the parameter.
                |             If the parameter is not locked, AdminLevel gives the administration
                |             level that will give the value of the parameter after a reset.
                |             
                |         ioLocked
                |             Indicates if the parameter has been locked. 
                | 
                |     Returns:
                |         Indicates if the parameter has been explicitly modified or remain to
                |         the administrated value.

        :param str io_admin_level:
        :param str io_locked:
        :rtype: bool
        """
        return self.work_general_setting_att.GetMoveActivityInfo(io_admin_level, io_locked)

    def get_style_sheet_path_info(self, io_admin_level: str, io_locked: str) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetStyleSheetPathInfo(CATBSTR ioAdminLevel,
                | CATBSTR ioLocked) As boolean
                | 
                |     Retrieves environment informations for the StyleSheetPath
                |     parameter.
                |     Role:Retrieves the state of the StyleSheetPath parameter in the current
                |     environment.
                | 
                |     Parameters:
                | 
                |         ioAdminLevel
                | 
                |             If the parameter is locked, AdminLevel gives the administration
                |             level that imposes the value of the parameter.
                |             If the parameter is not locked, AdminLevel gives the administration
                |             level that will give the value of the parameter after a reset.
                |             
                |         ioLocked
                |             Indicates if the parameter has been locked. 
                | 
                |     Returns:
                |         Indicates if the parameter has been explicitly modified or remain to
                |         the administrated value.

        :param str io_admin_level:
        :param str io_locked:
        :rtype: bool
        """
        return self.work_general_setting_att.GetStyleSheetPathInfo(io_admin_level, io_locked)

    def get_text_activity_info(self, io_admin_level: str, io_locked: str) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetTextActivityInfo(CATBSTR ioAdminLevel,
                | CATBSTR ioLocked) As boolean
                | 
                |     Retrieves environment informations for the TextActivity
                |     parameter.
                |     Role:Retrieves the state of the TextActivity parameter in the current
                |     environment.
                | 
                |     Parameters:
                | 
                |         ioAdminLevel
                | 
                |             If the parameter is locked, AdminLevel gives the administration
                |             level that imposes the value of the parameter.
                |             If the parameter is not locked, AdminLevel gives the administration
                |             level that will give the value of the parameter after a reset.
                |             
                |         ioLocked
                |             Indicates if the parameter has been locked. 
                | 
                |     Returns:
                |         Indicates if the parameter has been explicitly modified or remain to
                |         the administrated value.

        :param str io_admin_level:
        :param str io_locked:
        :rtype: bool
        """
        return self.work_general_setting_att.GetTextActivityInfo(io_admin_level, io_locked)

    def get_view_point_activity_info(self, io_admin_level: str, io_locked: str) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetViewPointActivityInfo(CATBSTR ioAdminLevel,
                | CATBSTR ioLocked) As boolean
                | 
                |     Retrieves environment informations for the ViewPointActivity
                |     parameter.
                |     Role:Retrieves the state of the ViewPointActivity parameter in the current
                |     environment.
                | 
                |     Parameters:
                | 
                |         ioAdminLevel
                | 
                |             If the parameter is locked, AdminLevel gives the administration
                |             level that imposes the value of the parameter.
                |             If the parameter is not locked, AdminLevel gives the administration
                |             level that will give the value of the parameter after a reset.
                |             
                |         ioLocked
                |             Indicates if the parameter has been locked. 
                | 
                |     Returns:
                |         Indicates if the parameter has been explicitly modified or remain to
                |         the administrated value.

        :param str io_admin_level:
        :param str io_locked:
        :rtype: bool
        """
        return self.work_general_setting_att.GetViewPointActivityInfo(io_admin_level, io_locked)

    def get_visibility_activity_info(self, io_admin_level: str, io_locked: str) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetVisibilityActivityInfo(CATBSTR ioAdminLevel,
                | CATBSTR ioLocked) As boolean
                | 
                |     Retrieves environment informations for the VisibilityActivity
                |     parameter.
                |     Role:Retrieves the state of the VisibilityActivity parameter in the current
                |     environment.
                | 
                |     Parameters:
                | 
                |         ioAdminLevel
                | 
                |             If the parameter is locked, AdminLevel gives the administration
                |             level that imposes the value of the parameter.
                |             If the parameter is not locked, AdminLevel gives the administration
                |             level that will give the value of the parameter after a reset.
                |             
                |         ioLocked
                |             Indicates if the parameter has been locked. 
                | 
                |     Returns:
                |         Indicates if the parameter has been explicitly modified or remain to
                |         the administrated value.

        :param str io_admin_level:
        :param str io_locked:
        :rtype: bool
        """
        return self.work_general_setting_att.GetVisibilityActivityInfo(io_admin_level, io_locked)

    def set_annotation_activity_lock(self, i_locked: bool) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetAnnotationActivityLock(boolean iLocked)
                | 
                |     Locks or unlocks the AnnotationActivity parameter.
                |     Role:Locks or unlocks the AnnotationActivity parameter if it is possible in
                |     the current administrative context. In user mode this method will always return
                |     E_FAIL.
                | 
                |     Parameters:
                | 
                |         iLocked
                |             the locking operation to be performed Legal
                |             values:
                |             TRUE : to lock the parameter.
                |             FALSE: to unlock the parameter.

        :param bool i_locked:
        :rtype: None
        """
        return self.work_general_setting_att.SetAnnotationActivityLock(i_locked)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_annotation_activity_lock'
        # # vba_code = """
        # # Public Function set_annotation_activity_lock(work_general_setting_att)
        # #     Dim iLocked (2)
        # #     work_general_setting_att.SetAnnotationActivityLock iLocked
        # #     set_annotation_activity_lock = iLocked
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_dnbwi_num_column_list_lock(self, i_locked: bool) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetDNBWINumColumnListLock(boolean iLocked)

        :param bool i_locked:
        :rtype: None
        """
        return self.work_general_setting_att.SetDNBWINumColumnListLock(i_locked)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_dnbwi_num_column_list_lock'
        # # vba_code = """
        # # Public Function set_dnbwi_num_column_list_lock(work_general_setting_att)
        # #     Dim iLocked (2)
        # #     work_general_setting_att.SetDNBWINumColumnListLock iLocked
        # #     set_dnbwi_num_column_list_lock = iLocked
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_move_activity_lock(self, i_locked: bool) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetMoveActivityLock(boolean iLocked)
                | 
                |     Locks or unlocks the MoveActivity parameter.
                |     Role:Locks or unlocks the MoveActivity parameter if it is possible in the
                |     current administrative context. In user mode this method will always return
                |     E_FAIL.
                | 
                |     Parameters:
                | 
                |         iLocked
                |             the locking operation to be performed Legal
                |             values:
                |             TRUE : to lock the parameter.
                |             FALSE: to unlock the parameter.

        :param bool i_locked:
        :rtype: None
        """
        return self.work_general_setting_att.SetMoveActivityLock(i_locked)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_move_activity_lock'
        # # vba_code = """
        # # Public Function set_move_activity_lock(work_general_setting_att)
        # #     Dim iLocked (2)
        # #     work_general_setting_att.SetMoveActivityLock iLocked
        # #     set_move_activity_lock = iLocked
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_style_sheet_path_lock(self, i_locked: bool) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetStyleSheetPathLock(boolean iLocked)
                | 
                |     Locks or unlocks the StyleSheetPath parameter.
                |     Role:Locks or unlocks the StyleSheetPath parameter if it is possible in the
                |     current administrative context. In user mode this method will always return
                |     E_FAIL.
                | 
                |     Parameters:
                | 
                |         iLocked
                |             the locking operation to be performed Legal
                |             values:
                |             TRUE : to lock the parameter.
                |             FALSE: to unlock the parameter.

        :param bool i_locked:
        :rtype: None
        """
        return self.work_general_setting_att.SetStyleSheetPathLock(i_locked)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_style_sheet_path_lock'
        # # vba_code = """
        # # Public Function set_style_sheet_path_lock(work_general_setting_att)
        # #     Dim iLocked (2)
        # #     work_general_setting_att.SetStyleSheetPathLock iLocked
        # #     set_style_sheet_path_lock = iLocked
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_text_activity_lock(self, i_locked: bool) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetTextActivityLock(boolean iLocked)
                | 
                |     Locks or unlocks the TextActivity parameter.
                |     Role:Locks or unlocks the TextActivity parameter if it is possible in the
                |     current administrative context. In user mode this method will always return
                |     E_FAIL.
                | 
                |     Parameters:
                | 
                |         iLocked
                |             the locking operation to be performed Legal
                |             values:
                |             TRUE : to lock the parameter.
                |             FALSE: to unlock the parameter.

        :param bool i_locked:
        :rtype: None
        """
        return self.work_general_setting_att.SetTextActivityLock(i_locked)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_text_activity_lock'
        # # vba_code = """
        # # Public Function set_text_activity_lock(work_general_setting_att)
        # #     Dim iLocked (2)
        # #     work_general_setting_att.SetTextActivityLock iLocked
        # #     set_text_activity_lock = iLocked
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_view_point_activity_lock(self, i_locked: bool) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetViewPointActivityLock(boolean iLocked)
                | 
                |     Locks or unlocks the ViewPointActivity parameter.
                |     Role:Locks or unlocks the ViewPointActivity parameter if it is possible in
                |     the current administrative context. In user mode this method will always return
                |     E_FAIL.
                | 
                |     Parameters:
                | 
                |         iLocked
                |             the locking operation to be performed Legal
                |             values:
                |             TRUE : to lock the parameter.
                |             FALSE: to unlock the parameter.

        :param bool i_locked:
        :rtype: None
        """
        return self.work_general_setting_att.SetViewPointActivityLock(i_locked)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_view_point_activity_lock'
        # # vba_code = """
        # # Public Function set_view_point_activity_lock(work_general_setting_att)
        # #     Dim iLocked (2)
        # #     work_general_setting_att.SetViewPointActivityLock iLocked
        # #     set_view_point_activity_lock = iLocked
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_visibility_activity_lock(self, i_locked: bool) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetVisibilityActivityLock(boolean iLocked)
                | 
                |     Locks or unlocks the VisibilityActivity parameter.
                |     Role:Locks or unlocks the VisibilityActivity parameter if it is possible in
                |     the current administrative context. In user mode this method will always return
                |     E_FAIL.
                | 
                |     Parameters:
                | 
                |         iLocked
                |             the locking operation to be performed Legal
                |             values:
                |             TRUE : to lock the parameter.
                |             FALSE: to unlock the parameter.

        :param bool i_locked:
        :rtype: None
        """
        return self.work_general_setting_att.SetVisibilityActivityLock(i_locked)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_visibility_activity_lock'
        # # vba_code = """
        # # Public Function set_visibility_activity_lock(work_general_setting_att)
        # #     Dim iLocked (2)
        # #     work_general_setting_att.SetVisibilityActivityLock iLocked
        # #     set_visibility_activity_lock = iLocked
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def __repr__(self):
        return f'WorkGeneralSettingAtt(name="{ self.name }")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from pycatia.cat_str_functional_interfaces.sfm_welds import SFMWelds
from pycatia.in_interfaces.reference import Reference
from pycatia.in_interfaces.references import References
from pycatia.system_interfaces.any_object import AnyObject
from pycatia.types.general import cat_variant


class SFMManager(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     SfmManager
                | 
                | Services about Structure Functional Modeler applications: SFD and
                | SDD.
                | Role: To manage some services.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.sfm_manager = com_object

    def add_hull(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub AddHull()
                | 
                |     Add the Hull feature according to the PRM resources.
                |     Role: Allows adding the Hull feature according to the PRM
                |     resources.
                | 
                |     Returns:
                |         S_OK if everything ran ok.

        :rtype: None
        """
        return self.sfm_manager.AddHull()

    def get_all_uc1_welds(self) -> SFMWelds:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetAllUC1Welds() As SfmWelds
                | 
                |     Gets all UC1 Weld features in Part.
                | 
                |     Parameters:
                | 
                |         oWelds
                |             [out] The retrieved UC1 Weld features. 
                | 
                |     Returns:
                |         S_OK if everything ran ok.
                | 
                |         Example :
                |             This example gets welds features in Part.
                | 
                |              Dim Welds As SfmWelds
                |              Set Welds = ManagerObj.GetWelds(Nothing)

        :rtype: SFMWelds
        """
        return SFMWelds(self.sfm_manager.GetAllUC1Welds())

    def get_hull(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetHull() As Reference
                | 
                |     Returns the Hull feature in the part.
                | 
                |     Example:
                |         This example retrieves in Hull the Hull object.
                | 
                |          Dim Hull As Reference
                |          SfmManager.GetHull Hull

        :rtype: Reference
        """
        return Reference(self.sfm_manager.GetHull())

    def get_plane_systems(self) -> References:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetPlaneSystems() As References
                | 
                |     Returns the list of PlaneSystems in the part.
                | 
                |     Example:
                |         This example retrieves in PlaneSystems the list of PlaneSystems
                |         objects.
                | 
                |          Dim PlaneSystems As References
                |          SfmManager.GetPlaneSystems PlaneSystems

        :rtype: References
        """
        return References(self.sfm_manager.GetPlaneSystems())

    def get_reference_plane(self, i_plane_system_index: cat_variant, i_plane_index: cat_variant) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetReferencePlane(CATVariant iPlaneSystemIndex,
                | CATVariant iPlaneIndex) As Reference
                | 
                |     Returns a reference plane in the specific PlaneSystems.
                | 
                |     Example:
                |         This example retrieves in RefPlane the reference planes contained into
                |         the 1st PlaneSystem, and which has CROSS.12 as a name.
                | 
                |          Dim RefPlane As Reference
                |          SfmManager.GetReferencePlane 1, "CROSS.12", RefPlane

        :param cat_variant i_plane_system_index:
        :param cat_variant i_plane_index:
        :rtype: Reference
        """
        return Reference(
            self.sfm_manager.GetReferencePlane(
                i_plane_system_index,
                i_plane_index
            )
        )

    def get_super_members(self) -> References:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetSuperMembers() As References
                | 
                |     Returns the list of SuperMembers in the part.
                | 
                |     Example:
                |         This example retrieves in SuperMembers the list of SuperMembers
                |         objects.
                | 
                |          Dim SuperMembers As References
                |          SfmManager.GetSuperMembers SuperMembers

        :rtype: References
        """
        return References(self.sfm_manager.GetSuperMembers())

    def get_super_plates(self) -> References:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetSuperPlates() As References
                | 
                |     Returns the list of SuperPlates in the part.
                | 
                |     Example:
                |         This example retrieves in SuperPlates the list of SuperPlates
                |         objects.
                | 
                |          Dim SuperPlates As References
                |          SfmManager.GetSuperPlates SuperPlates

        :rtype: References
        """
        return References(self.sfm_manager.GetSuperPlates())

    def get_super_stiffeners(self) -> References:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetSuperStiffeners() As References
                | 
                |     Returns the list of SuperStiffeners in the part.
                | 
                |     Example:
                |         This example retrieves in SuperStiffeners the list of SuperStiffeners
                |         objects.
                | 
                |          Dim SuperStiffeners As References
                |          SfmManager.GetSuperStiffeners SuperStiffeners

        :rtype: References
        """
        return References(self.sfm_manager.GetSuperStiffeners())

    def get_super_stiffeners_on_free_edge(self) -> References:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetSuperStiffenersOnFreeEdge() As References
                | 
                |     Returns the list of SuperStiffenersOnFreeEdge in the part.
                | 
                |     Example:
                |         This example retrieves in SuperStiffenersOnFreeEdge the list of
                |         SuperStiffenersOnFreeEdge objects.
                | 
                |          Dim SuperStiffenersOnFreeEdge As References
                |          SfmManager.GetSuperStiffenersOnFreeEdge
                |          SuperStiffenersOnFreeEdge

        :rtype: References
        """
        return References(self.sfm_manager.GetSuperStiffenersOnFreeEdge())

    def init_resources(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub InitResources()
                | 
                |     Initialize environment (PRM resources).
                |     Role: Allows initializing environment (PRM resources).
                | 
                |     Returns:
                |         S_OK if everything ran ok.

        :rtype: None
        """
        return self.sfm_manager.InitResources()

    def synchronize_hull(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SynchronizeHull()
                | 
                |     Synchronize PlaneSystems according with the PRM resources.
                |     Role: Allows synchronizing the PlaneSystems with the PRM
                |     resources.
                | 
                |     Returns:
                |         S_OK if everything ran ok.

        :rtype: None
        """
        return self.sfm_manager.SynchronizeHull()

    def synchronize_planes(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SynchronizePlanes()
                | 
                |     Synchronize PlaneSystems according with the PRM resources.
                |     Role: Allows synchronizing the PlaneSystems with the PRM
                |     resources.
                | 
                |     Returns:
                |         S_OK if everything ran ok.

        :rtype: None
        """
        return self.sfm_manager.SynchronizePlanes()

    def __repr__(self):
        return f'SFMManager(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-07-06 14:02:20.222384

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.hybrid_shape_interfaces.hybrid_shape_direction import HybridShapeDirection
from pycatia.in_interfaces.reference import Reference
from pycatia.mec_mod_interfaces.hybrid_shape import HybridShape


class HybridShapeAxisLine(HybridShape):

    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.HybridShape
                |                         HybridShapeAxisLine
                | 
                | Represents the hybrid shape axis line feature object.
                | Role: To access the data of the hybrid shape axis line feature object. This
                | data includes:
                | 
                |     The element used to compute the axis
                |     The direction used in orientation of axis
                |     AxisLineType to change the axis type
                | 
                | Use the CATIAHybridShapeFactory to create a HybridShapeAxisLine
                | object.
                | 
                | See also:
                |     HybridShapeFactory
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.hybrid_shape_axis_line = com_object

    @property
    def axis_line_type(self) -> int:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property AxisLineType() As long
                | 
                |     Returns or sets the axis line type.
                |     Legal values:
                | 
                |     1
                |         This option creates Axis along major axis if element is ellipse or
                |         oblong, Axis is aligned with direction specified if input is circle and
                |         coincides with revolution axis if element is revolution
                |         surface
                |     2
                |         This option creates Axis along minor axis if element is ellipse or
                |         oblong, Axis is normal to direction specified if input is
                |         circle
                |     3
                |         This option creates Axis normal to the element if it is circle, ellipse
                |         or oblong
                | 
                | Example:
                |     This example retrieves in oType the axis line type for the AxisLine hybrid
                |     shape feature.
                | 
                |      Dim oType
                |      Set oType = AxisLine.AxisLineType

        :rtype: int
        """

        return self.hybrid_shape_axis_line.AxisLineType

    @axis_line_type.setter
    def axis_line_type(self, value: int):
        """
        :param int value:
        """

        self.hybrid_shape_axis_line.AxisLineType = value

    @property
    def direction(self) -> HybridShapeDirection:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Direction() As HybridShapeDirection
                | 
                |     Gets the reference direction used in computation of axis.
                |     This is useful only if the element selected is circle, arc or sphere. If
                |     the element is circle or arc Axis may be normal to reference direction or
                |     aligned with reference direction
                | 
                |     Example:
                |         This example retrieves in oDir the direction for the AxisLine hybrid
                |         shape feature.
                | 
                |          Dim oDir As CATIAHybridShapeDirection
                |          Set oDir = AxisLine.Direction

        :rtype: HybridShapeDirection
        """

        return HybridShapeDirection(self.hybrid_shape_axis_line.Direction)

    @direction.setter
    def direction(self, value: HybridShapeDirection):
        """
        :param HybridShapeDirection value:
        """

        self.hybrid_shape_axis_line.Direction = value

    @property
    def element(self) -> Reference:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-07-06 14:02:20.222384)
                | o Property Element() As Reference
                | 
                |     Returns or sets the element from which axis is computed.
                | 
                |     Example:
                |         This example retrieves in Element the element from which axis is
                |         computed for the AxisLine hybrid shape feature.
                | 
                |          Dim Element As Reference 
                |          Set Element = AxisLine.Element

        :rtype: Reference
        """

        return Reference(self.hybrid_shape_axis_line.Element)

    @element.setter
    def element(self, value: Reference):
        """
        :param Reference value:
        """

        self.hybrid_shape_axis_line.Element = value.com_object

    def __repr__(self):
        return f'HybridShapeAxisLine(name="{ self.name }")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from pycatia.cat_sch_platform_interfaces.sch_grr import SchGRR
from pycatia.cat_sch_platform_interfaces.sch_list_of_doubles import SchListOfDoubles
from pycatia.cat_sch_platform_interfaces.sch_list_of_objects import SchListOfObjects
from pycatia.cat_sch_platform_interfaces.sch_route_symbol import SchRouteSymbol
from pycatia.system_interfaces.any_object import AnyObject


class SchGRRRoute(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     SchGRRRoute
                | 
                | Manage the graphical representations of a schematic route.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.sch_grr_route = com_object

    def add_points(self, i_l_db2_pt_path_to_add: tuple, i_after_which_pt_num: int) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub AddPoints(CATSafeArrayVariant iLDb2PtPathToAdd,
                | long iAfterWhichPtNum)
                | 
                |     Add a list of point to a route.
                | 
                |     Parameters:
                | 
                |         iLDbPtPathToAdd
                |             A list of X-Y coordinates of the points to be added. 2 doubles per
                |             point. 
                |         iAfterWhichPtNum
                |             The point number to add the points after. Use 0 to indicate adding
                |             before the first point. 
                | 
                |     Example:
                |
                |          Dim objThisIntf As SchGRRRoute
                |          Dim dbVar1(x) As CATSafeArrayVariant
                |          Dim intVar3 As Integer
                |           ...
                |          objThisIntf.AddPointsdbVar1,intVar3

        :param tuple i_l_db2_pt_path_to_add:
        :param int i_after_which_pt_num:
        :rtype: tuple
        """
        return self.sch_grr_route.AddPoints(i_l_db2_pt_path_to_add, i_after_which_pt_num)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'add_points'
        # # vba_code = """
        # # Public Function add_points(sch_grr_route)
        # #     Dim iLDb2PtPathToAdd (2)
        # #     sch_grr_route.AddPoints iLDb2PtPathToAdd
        # #     add_points = iLDb2PtPathToAdd
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def break_(self, i_db2_pt1: tuple, i_db2_pt2: tuple, o_new_grr_route: 'SchGRRRoute') -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub Break(CATSafeArrayVariant iDb2Pt1,
                | CATSafeArrayVariant iDb2Pt2,
                | SchGRRRoute oNewGRRRoute)
                | 
                |     Break a route graphic into 2 pieces. The old graphic is shortened and a new
                |     graphic is created.
                | 
                |     Parameters:
                | 
                |         iDb2Pt1
                |             X-Y coordinates of point 1 to break the route at (this point is
                |             mandatory). 
                |         iDb2Pt2
                |             X-Y coordinates of point 2 to break the route at (this point is
                |             optional). If provided the points in between point 1 and this point will be
                |             eliminated. Point 1 is the last point of the shortened old route and point 2 is
                |             the first point of the new route. If this point is not provided (i.e. sends in
                |             a NULL). point 1 and point 2 are the same. 
                |         oNewGRRRoute
                |             The new line string graphic created (CATISchGRRRoute interface
                |             pointer) 
                | 
                |     Example:
                | 
                |           
                | 
                |          Dim objThisIntf As SchGRRRoute
                |          Dim dbVar1(2) As CATSafeArrayVariant
                |          Dim dbVar2(2) As CATSafeArrayVariant
                |          Dim objArg3 As SchGRRRoute
                |           ...
                |          objThisIntf.BreakdbVar1,dbVar2,objArg3

        :param tuple i_db2_pt1:
        :param tuple i_db2_pt2:
        :param SchGRRRoute o_new_grr_route:
        :rtype: tuple
        """
        return self.sch_grr_route.Break(i_db2_pt1, i_db2_pt2, o_new_grr_route.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'break'
        # # vba_code = """
        # # Public Function break(sch_grr_route)
        # #     Dim iDb2Pt1 (2)
        # #     sch_grr_route.Break iDb2Pt1
        # #     break = iDb2Pt1
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def compress(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub Compress()
                | 
                |     Compress a the defining points of a route graphic, removing coincident
                |     points.
                | 
                |     Example:
                | 
                |           
                | 
                |          Dim objThisIntf As SchGRRRoute
                |           ...
                |          objThisIntf.Compress

        :rtype: None
        """
        return self.sch_grr_route.Compress()

    def compress2(self, i_unset_gaps: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub Compress2(CatSchIDLRouteUnsetGapsMode iUnsetGaps)
                | 
                |     Compress the defining points of a route graphic, removing coincident
                |     points.
                | 
                |     Parameters:
                | 
                |         iUnsetGaps
                |             Whether to unset gaps (in all the effected routes: this route
                |             and other routes intersecting it) or not = SchUnsetGapsOn :
                |             unset gaps = SchUnsetGapsOff : don't unset gaps
                | 
                |     Example:
                |
                |          Dim objThisIntf As SchGRRRoute
                | 
                |           ...
                |          objThisIntf.Compress2CatSchIDLRouteUnsetGapsMode_Enum

        :param int i_unset_gaps: enum cat_sch_idl_route_unset_gaps_mode
        :rtype: None
        """
        return self.sch_grr_route.Compress2(i_unset_gaps)

    def concatenate(self, i_which_end1: int, i_grr_route2: 'SchGRRRoute', i_which_end2: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub Concatenate(long iWhichEnd1,
                | SchGRRRoute iGRRRoute2,
                | long iWhichEnd2)
                | 
                |     Concatenate 2 route graphic objects into one. The first route graphic is
                |     elongated and the second object is deleted.
                | 
                |     Parameters:
                | 
                |         iWhichEnd1
                |             =1 at start point; =2 at end point 
                |         iGRRRoute2
                |             Second route graphic object (CATISchGRRRoute interface pointer) to
                |             be concatenated to the first. This route graphic will be deleted.
                |             
                |         iWhichEnd2
                |             =1 at start point; =2 at end point 
                | 
                |     Example:
                |
                |          Dim objThisIntf As SchGRRRoute
                |          Dim intVar1 As Integer
                |          Dim objArg2 As SchGRRRoute
                |          Dim intVar3 As Integer
                |           ...
                |          objThisIntf.ConcatenateintVar1,objArg2,intVar3

        :param int i_which_end1:
        :param SchGRRRoute i_grr_route2:
        :param int i_which_end2:
        :rtype: None
        """
        return self.sch_grr_route.Concatenate(i_which_end1, i_grr_route2.com_object, i_which_end2)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'concatenate'
        # # vba_code = """
        # # Public Function concatenate(sch_grr_route)
        # #     Dim iWhichEnd1 (2)
        # #     sch_grr_route.Concatenate iWhichEnd1
        # #     concatenate = iWhichEnd1
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def concatenate_keep_grr2(self, i_which_end1: int, i_grr_route2: 'SchGRRRoute', i_which_end2: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub ConcatenateKeepGRR2(long iWhichEnd1,
                | SchGRRRoute iGRRRoute2,
                | long iWhichEnd2)
                | 
                |     Concatenate 2 route graphic objects into one. The first route graphic is
                |     elongated and the second object is unchanged.
                | 
                |     Parameters:
                | 
                |         iWhichEnd1
                |             =1 at start point; =2 at end point 
                |         iGRRRoute2
                |             Second route graphic object (CATISchGRRRoute interface pointer) to
                |             be concatenated to the first. This route graphic will be unchanged.
                |             
                |         iWhichEnd2
                |             =1 at start point; =2 at end point 
                | 
                |     Example:
                |
                |          Dim objThisIntf As SchGRRRoute
                |          Dim intVar1 As Integer
                |          Dim objArg2 As SchGRRRoute
                |          Dim intVar3 As Integer
                |           ...
                |         
                |         objThisIntf.ConcatenateKeepGRR2intVar1,objArg2,intVar3

        :param int i_which_end1:
        :param SchGRRRoute i_grr_route2:
        :param int i_which_end2:
        :rtype: None
        """
        return self.sch_grr_route.ConcatenateKeepGRR2(i_which_end1, i_grr_route2.com_object, i_which_end2)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'concatenate_keep_grr2'
        # # vba_code = """
        # # Public Function concatenate_keep_grr2(sch_grr_route)
        # #     Dim iWhichEnd1 (2)
        # #     sch_grr_route.ConcatenateKeepGRR2 iWhichEnd1
        # #     concatenate_keep_grr2 = iWhichEnd1
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def create_route_symbol(
            self,
            i_seg_num: int,
            i_seg_parm: float,
            i_grr_symbol: SchGRR,
            o_route_symbol: SchRouteSymbol
    ) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub CreateRouteSymbol(long iSegNum,
                | double iSegParm,
                | SchGRR iGRRSymbol,
                | SchRouteSymbol oRouteSymbol)
                | 
                |     Place a symbol on the route.
                | 
                |     Parameters:
                | 
                |         iSegNum
                |             The route segment number to place the symbol on. 
                |         iSegParm
                |             The parameter along the segment used to place the symbol on
                |             (0.<=iSegParm<=1.). 
                |         iGRRSymbol
                |             The graphical primitive (detail) to be used for the symbol.
                |             
                |         oRouteSymbol
                |             The created route symbol (ditto). 
                | 
                |     Example:
                |
                |          Dim objThisIntf As SchGRRRoute
                |          Dim intVar1 As Integer
                |          Dim dbVar2 As Double;
                |          Dim objArg3 As SchGRR
                |          Dim objArg4 As SchRouteSymbol
                |           ...
                |          objThisIntf.CreateRouteSymbolintVar1,dbVar2,objArg3,objArg4

        :param int i_seg_num:
        :param float i_seg_parm:
        :param SchGRR i_grr_symbol:
        :param SchRouteSymbol o_route_symbol:
        :rtype: None
        """
        return self.sch_grr_route.CreateRouteSymbol(
            i_seg_num,
            i_seg_parm,
            i_grr_symbol.com_object,
            o_route_symbol.com_object
        )
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'create_route_symbol'
        # # vba_code = """
        # # Public Function create_route_symbol(sch_grr_route)
        # #     Dim iSegNum (2)
        # #     sch_grr_route.CreateRouteSymbol iSegNum
        # #     create_route_symbol = iSegNum
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_end_point(self, o_db2_end_pt: SchListOfDoubles) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub GetEndPoint(SchListOfDoubles oDb2EndPt)
                | 
                |     Get the end point of the route graphic.
                | 
                |     Parameters:
                | 
                |         oDb2EndPt
                |             X-Y coordinates of the end point of the route graphic object.
                |             
                |     Example:
                |
                |          Dim objThisIntf As SchGRRRoute
                |          Dim objArg1 As SchListOfDoubles
                |           ...
                |          objThisIntf.GetEndPointobjArg1

        :param SchListOfDoubles o_db2_end_pt:
        :rtype: None
        """
        return self.sch_grr_route.GetEndPoint(o_db2_end_pt.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'get_end_point'
        # # vba_code = """
        # # Public Function get_end_point(sch_grr_route)
        # #     Dim oDb2EndPt (2)
        # #     sch_grr_route.GetEndPoint oDb2EndPt
        # #     get_end_point = oDb2EndPt
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_path(self, o_l_db2_pt_path: SchListOfDoubles) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub GetPath(SchListOfDoubles oLDb2PtPath)
                | 
                |     Get the defining points of a route graphic.
                | 
                |     Parameters:
                | 
                |         oLDbPtPath
                |             A list of X-Y coordinates of the points. 2 doubles per point.
                |
                |     Example:
                |
                |          Dim objThisIntf As SchGRRRoute
                |          Dim objArg1 As SchListOfDoubles
                |          Dim intVar2 As Integer
                |           ...
                |          objThisIntf.GetPathobjArg1

        :param SchListOfDoubles o_l_db2_pt_path:
        :rtype: None
        """
        return self.sch_grr_route.GetPath(o_l_db2_pt_path.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'get_path'
        # # vba_code = """
        # # Public Function get_path(sch_grr_route)
        # #     Dim oLDb2PtPath (2)
        # #     sch_grr_route.GetPath oLDb2PtPath
        # #     get_path = oLDb2PtPath
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def get_start_point(self, o_db2_start_pt: SchListOfDoubles) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub GetStartPoint(SchListOfDoubles oDb2StartPt)
                | 
                |     Get the start point of the route graphic.
                | 
                |     Parameters:
                | 
                |         oDb2StartPt
                |             X-Y coordinates of the start point of the route graphic object.
                |
                |     Example:
                |
                |          Dim objThisIntf As SchGRRRoute
                |          Dim objArg1 As SchListOfDoubles
                |           ...
                |          objThisIntf.GetStartPointobjArg1

        :param SchListOfDoubles o_db2_start_pt:
        :rtype: None
        """
        return self.sch_grr_route.GetStartPoint(o_db2_start_pt.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'get_start_point'
        # # vba_code = """
        # # Public Function get_start_point(sch_grr_route)
        # #     Dim oDb2StartPt (2)
        # #     sch_grr_route.GetStartPoint oDb2StartPt
        # #     get_start_point = oDb2StartPt
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def list_route_symbols(self) -> SchListOfObjects:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func ListRouteSymbols() As SchListOfObjects
                | 
                |     List route symbols on the route.
                | 
                |     Parameters:
                | 
                |         oLRouteSymbol
                |             A list of route symbols. (members are CATIASchRouteSymbol objects).
                |
                |     Example:
                |
                |          Dim objThisIntf As SchGRRRoute
                |          Dim objArg1 As SchListOfObjects
                |           ...
                |          Set objArg1 = objThisIntf.ListRouteSymbols

        :rtype: SchListOfObjects
        """
        return SchListOfObjects(self.sch_grr_route.ListRouteSymbols())

    def remove_points(self, i_num_of_pts_to_remove: int, i_after_which_pt_num: int) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub RemovePoints(long iNumOfPtsToRemove,
                | long iAfterWhichPtNum)
                | 
                |     Remove points from route graphic.
                | 
                |     Parameters:
                | 
                |         iNumOfPtsToRemove
                |             The number of points to be removed 
                |         iAfterWhichPtNum
                |             The point number at which to start removing the point.
                |
                |     Example:
                |
                |          Dim objThisIntf As SchGRRRoute
                |          Dim intVar1 As Integer
                |          Dim intVar2 As Integer
                |           ...
                |          objThisIntf.RemovePointsintVar1,intVar2

        :param int i_num_of_pts_to_remove:
        :param int i_after_which_pt_num:
        :rtype: None
        """
        return self.sch_grr_route.RemovePoints(i_num_of_pts_to_remove, i_after_which_pt_num)

    def set_end_point(self, i_db2_end_pt: tuple) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetEndPoint(CATSafeArrayVariant iDb2EndPt)
                | 
                |     Set the end point of the route graphic.
                | 
                |     Parameters:
                | 
                |         iDb2EndPt
                |             X-Y coordinates of the end point of the route graphic object to be
                |             set. 
                | 
                |     Example:
                |
                |          Dim objThisIntf As SchGRRRoute
                |          Dim dbVar1(2) As CATSafeArrayVariant
                |           ...
                |          objThisIntf.SetEndPointdbVar1

        :param tuple i_db2_end_pt:
        :rtype: tuple
        """
        return self.sch_grr_route.SetEndPoint(i_db2_end_pt)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_end_point'
        # # vba_code = """
        # # Public Function set_end_point(sch_grr_route)
        # #     Dim iDb2EndPt (2)
        # #     sch_grr_route.SetEndPoint iDb2EndPt
        # #     set_end_point = iDb2EndPt
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_path(self, i_l_db2_pt_path: tuple, i_compress: int) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetPath(CATSafeArrayVariant iLDb2PtPath,
                | CatSchIDLRouteCompressMode iCompress)
                | 
                |     Set the defining points of a route graphic.
                | 
                |     Parameters:
                | 
                |         iLDbPtPath
                |             A list of X-Y coordinates of the points to be set. 2 doubles per
                |             point. 
                |         iCompress
                |             Whether to compress the route (i.e., remove duplicate pts, colinear segments, etc.) or
                |             not = SchCompressOn : compress = SchCompressOff : don't compress
                | 
                |     Example:
                |
                |          Dim objThisIntf As SchGRRRoute
                |          Dim dbVar1(x) As CATSafeArrayVariant
                | 
                |           ...
                |          objThisIntf.SetPathdbVar1,CatSchIDLRouteCompressMode_Enum

        :param tuple i_l_db2_pt_path:
        :param int i_compress: enum cat_sch_idl_route_compress_mode
        :rtype: tuple
        """
        return self.sch_grr_route.SetPath(i_l_db2_pt_path, i_compress)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_path'
        # # vba_code = """
        # # Public Function set_path(sch_grr_route)
        # #     Dim iLDb2PtPath (2)
        # #     sch_grr_route.SetPath iLDb2PtPath
        # #     set_path = iLDb2PtPath
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_path2(self, i_l_db2_pt_path: tuple, i_compress: int, i_unset_gaps: int) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetPath2(CATSafeArrayVariant iLDb2PtPath,
                | CatSchIDLRouteCompressMode iCompress,
                | CatSchIDLRouteUnsetGapsMode iUnsetGaps)
                | 
                |     Set the defining points of a route graphic.
                | 
                |     Parameters:
                | 
                |         iLDbPtPath
                |             A list of X-Y coordinates of the points to be set. 2 doubles per
                |             point. 
                |         iCompress
                |             Whether to compress the route (i.e., remove duplicate pts, colinear segments, etc.)
                |             or not = SchCompressOn : compress = SchCompressOff : don't compress
                |         iUnsetGaps
                |             Whether to unset gaps (in all the effected routes: this route and other routes
                              intersecting it) or not = SchUnsetGapsOn : unset gaps = SchUnsetGapsOff : don't unset gaps
                | 
                |     Example:
                |
                |          Dim objThisIntf As SchGRRRoute
                |          Dim dbVar1(x) As CATSafeArrayVariant
                | 
                |           ...
                |          objThisIntf.SetPath2dbVar1,CatSchIDLRouteCompressMode_Enum,CatSchIDLRouteUnsetGapsMode_Enum

        :param tuple i_l_db2_pt_path:
        :param int i_compress: enum cat_sch_idl_route_compress_mode
        :param int i_unset_gaps: enum cat_sch_idl_route_unset_gaps_mode
        :rtype: tuple
        """
        return self.sch_grr_route.SetPath2(i_l_db2_pt_path, i_compress, i_unset_gaps)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_path2'
        # # vba_code = """
        # # Public Function set_path2(sch_grr_route)
        # #     Dim iLDb2PtPath (2)
        # #     sch_grr_route.SetPath2 iLDb2PtPath
        # #     set_path2 = iLDb2PtPath
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_path3(self, i_l_db2_pt_path: tuple, i_compress: int, i_unset_gaps: int,
                  i_route_update_symbols: int) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetPath3(CATSafeArrayVariant iLDb2PtPath,
                | CatSchIDLRouteCompressMode iCompress,
                | CatSchIDLRouteUnsetGapsMode iUnsetGaps,
                | CatSchIDLRouteSymbolUpdateMode iRouteUpdateSymbols)
                | 
                |     Set the defining points of a route graphic.
                | 
                |     Parameters:
                | 
                |         iLDb2PtPath
                |             A list of X-Y coordinates of the points to be set. 2 doubles per
                |             point. 
                |         iCompress
                |             Whether to compress the route (i.e., remove duplicate pts, colinear segments, etc.)
                |             or not = catSchIDLCompressOn : compress = catSchIDLCompressOff : don't compress
                |         iUnsetGaps
                |             Whether to unset gaps (in all the effected routes: this route and other routes
                |             intersecting it) or not = catSchIDLUnsetGapsOn : unset gaps = catSchIDLUnsetGapsOff :
                |             don't unset gaps
                |         iRouteSymbolUpdate
                |             Whether to update route symbols' positions = catSchIDLSymbolUpdateOff : don't update
                |             route symbols = catSchIDLSymbolUpdateOn : update route symbols
                | 
                |     Example:
                |
                |          Dim objThisIntf As SchGRRRoute
                |          Dim dbVar1(x) As CATSafeArrayVariant
                | 
                |           ...
                |          objThisIntf.SetPath3dbVar1,CatSchIDLRouteCompressMode_Enum,CatSchIDLRouteUnsetGapsMode_Enum,CatSchIDLRouteSymbolUpdateMode_Enum

        :param tuple i_l_db2_pt_path:
        :param int i_compress: enum cat_sch_idl_route_compress_mode
        :param int i_unset_gaps: enum cat_sch_idl_route_unset_gaps_mode
        :param int i_route_update_symbols: enum cat_sch_idl_route_symbol_update_mode
        :rtype: tuple
        """
        return self.sch_grr_route.SetPath3(i_l_db2_pt_path, i_compress, i_unset_gaps, i_route_update_symbols)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_path3'
        # # vba_code = """
        # # Public Function set_path3(sch_grr_route)
        # #     Dim iLDb2PtPath (2)
        # #     sch_grr_route.SetPath3 iLDb2PtPath
        # #     set_path3 = iLDb2PtPath
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def set_start_point(self, i_db2_start_pt: tuple) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetStartPoint(CATSafeArrayVariant iDb2StartPt)
                | 
                |     Set the start point of the route graphic.
                | 
                |     Parameters:
                | 
                |         iDb2StartPt
                |             X-Y coordinates of the start point of the route graphic object to
                |             be set. 
                | 
                |     Example:
                | 
                |           
                | 
                |          Dim objThisIntf As SchGRRRoute
                |          Dim dbVar1(2) As CATSafeArrayVariant
                |           ...
                |          objThisIntf.SetStartPointdbVar1

        :param tuple i_db2_start_pt:
        :rtype: tuple
        """
        return self.sch_grr_route.SetStartPoint(i_db2_start_pt)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_start_point'
        # # vba_code = """
        # # Public Function set_start_point(sch_grr_route)
        # #     Dim iDb2StartPt (2)
        # #     sch_grr_route.SetStartPoint iDb2StartPt
        # #     set_start_point = iDb2StartPt
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def __repr__(self):
        return f'SchGRRRoute(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from pycatia.cat_plant_ship_interfaces.psp_list_of_doubles import PSPListOfDoubles
from pycatia.product_structure_interfaces.product import Product
from pycatia.system_interfaces.any_object import AnyObject


class PSPLightPart(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     PspLightPart
                | 
                | Represents the light part.
                | Role: To access light object data.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.psp_light_part = com_object

    def get_definition(self, i_rel_axis: Product) -> PSPListOfDoubles:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetDefinition(Product iRelAxis) As PspListOfDoubles
                | 
                |     Retrieves the points defining the object.
                | 
                |     Parameters:
                | 
                |         iRelAxis
                |             The relative axis object (Nothing means relative to parent).
                |             
                | 
                |     Returns:
                |         List of points defining object. A list of X-Y-Z coordinates of the
                |         points. 3 doubles per point.
                |
                |     Example:
                |
                |          Dim objThisIntf As PspLightPart
                |          Dim objArg1 As Product
                |          Dim objArg2 As PspListOfDoubles
                |          Set objArg1 = Nothing
                |           ...
                |          Set objArg2 = objThisIntf.GetDefinition (objArg1)

        :param Product i_rel_axis:
        :rtype: PSPListOfDoubles
        """
        return PSPListOfDoubles(self.psp_light_part.GetDefinition(i_rel_axis.com_object))

    def set_definition(self, i_rel_axis: Product, i_list_points: tuple) -> tuple:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub SetDefinition(Product iRelAxis,
                | CATSafeArrayVariant iListPoints)
                | 
                |     Set the points defining the light object.
                | 
                |     Parameters:
                | 
                |         iRelAxis
                |             The relative axis object (Nothing means relative to parent).
                |             
                |         oListPoints
                |             List of points defining object. A list of X-Y-Z coordinates of the
                |             points. 3 doubles per point. 
                | 
                |     Example:
                |
                |          Dim objThisIntf As PspLightPart
                |          Dim objArg1 As CATSafeArrayVariant
                |          Dim dbVar2(3) As CATSafeArrayVariant
                |           ...
                |          objThisIntf.SetDefinition objArg1, dbVar2

        :param Product i_rel_axis:
        :param tuple i_list_points:
        :rtype: tuple
        """
        return self.psp_light_part.SetDefinition(i_rel_axis.com_object, i_list_points)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'set_definition'
        # # vba_code = """
        # # Public Function set_definition(psp_light_part)
        # #     Dim iRelAxis (2)
        # #     psp_light_part.SetDefinition iRelAxis
        # #     set_definition = iRelAxis
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def __repr__(self):
        return f'PSPLightPart(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from typing import TYPE_CHECKING

from pycatia.dmaps_interfaces.activities import Activities
from pycatia.dmaps_interfaces.items import Items
from pycatia.dmaps_interfaces.outputs import Outputs
from pycatia.system_interfaces.any_object import AnyObject

if TYPE_CHECKING:
    from pycatia.dmaps_interfaces.resource_collection import ResourceCollection


class Resource(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     Resource
                | 
                | The object that represents a resource.
                | 
                | It also allows to get the real resource object.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.resource = com_object

    @property
    def children_ts_as(self) -> Activities:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property ChildrenTSAs() As Activities (Read Only)
                | 
                |     This property returns the interface which manages the children TSA of the
                |     given resource/system.

        :rtype: Activities
        """

        return Activities(self.resource.ChildrenTSAs)

    @property
    def input_products(self) -> Items:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property InputProducts() As Items (Read Only)
                | 
                |     This property returns the interface which manages the items or input
                |     products/components assigned to the given resource/system

        :rtype: Items
        """

        return Items(self.resource.InputProducts)

    @property
    def next_resources(self) -> 'ResourceCollection':
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property NextResources() As ResourceCollection (Read Only)
                | 
                |     This property returns the interface which manages the downstream product
                |     flow hierarchy on the given resource. This returns the collection of resources
                |     which are "Next" to the given resource in the resource product flow links

        :rtype: ResourceCollection
        """

        from pycatia.dmaps_interfaces.resource_collection import ResourceCollection
        return ResourceCollection(self.resource.NextResources)

    @property
    def output_products(self) -> Outputs:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property OutputProducts() As Outputs (Read Only)
                | 
                |     This property returns the interface which manages the output
                |     products/components of the given resource/system

        :rtype: Outputs
        """

        return Outputs(self.resource.OutputProducts)

    @property
    def previous_resources(self) -> 'ResourceCollection':
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property PreviousResources() As ResourceCollection (Read
                | Only)
                | 
                |     This property returns the interface which manages the upstream product flow
                |     hierarchy on the given resource. This returns the collection of resources which
                |     are "Previous" to the given resource in the resource product flow links

        :rtype: ResourceCollection
        """
        
        from pycatia.dmaps_interfaces.resource_collection import ResourceCollection
        return ResourceCollection(self.resource.PreviousResources)

    def get_object(self, i_interface_name: str) -> AnyObject:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetObject(CATBSTR iInterfaceName) As AnyObject
                | 
                |     This method returns the IDL Interface for the specified interface
                |     identifier.
                | 
                |     Parameters:
                | 
                |         iInterfaceName
                |             The name of the interface to query. 
                |         oObject
                |             The pointer to the queried interface.

        :param str i_interface_name:
        :rtype: AnyObject
        """
        return AnyObject(self.resource.GetObject(i_interface_name))

    def __repr__(self):
        return f'Resource(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""

from pycatia.system_interfaces.any_object import AnyObject


class SchPostReplace(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     SchPostReplace
                | 
                | Manage post component replacement behaviors.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.sch_post_replace = com_object

    def post_replace_text(self, i_new_sch_object: AnyObject) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub PostReplaceText(AnyObject iNewSchObject)
                | 
                |     Handle associated annotations after replace.
                | 
                |     Parameters:
                | 
                |         iNewSchObject
                |             Pointer to the object that is replacing 'this' object.
                |             
                |     Example:
                |
                |          Dim objThisIntf As SchPostReplace
                |          Dim objArg1 As AnyObject
                |           ...
                |          objThisIntf.PostReplaceTextobjArg1

        :param AnyObject i_new_sch_object:
        :rtype: None
        """
        return self.sch_post_replace.PostReplaceText(i_new_sch_object.com_object)

    def __repr__(self):
        return f'SchPostReplace(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from pycatia.dnb_igp_setup_interfaces.robot_task import RobotTask
from pycatia.system_interfaces.any_object import AnyObject


class RobotTaskCloneFactory(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     RobotTaskCloneFactory
                | 
                | Represents the Clone Factory for Robot Tasks.
                | 
                | Role: Robot Clone Task Factory is the object used to clone robot
                | tasks.
                | The following code snippet can be used to obtain the Robot Task Factory from
                | the robot product.
                | 
                |    Dim objRobotTaskFactory As RobotTaskCloneFactory
                |    Dim objRobot As Product
                |    ...
                |    Set objRobotTaskCloneFactory = objRobot.GetTechnologicalObject("RobotTaskCloneFactory" )
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.robot_task_clone_factory = com_object

    def clone_task_in_another_robot(
            self,
            i_src_robot_task: RobotTask,
            i_dest_robot: AnyObject,
            i_target_task_name: str,
            i_trans_on_target: tuple,
            o_cloned_robot_task: RobotTask) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub CloneTaskInAnotherRobot(RobotTask iSrcRobotTask,
                | AnyObject iDestRobot,
                | CATBSTR iTargetTaskName,
                | CATSafeArrayVariant iTransOnTarget,
                | RobotTask oClonedRobotTask)
                | 
                |     Clones a given Robot Task with in another robot.
                | 
                |     Parameters:
                | 
                |         iSrcRobotTask
                |             The Robot task that needs to be cloned. 
                |         iTargetTaskName
                |             The name for the cloned robot task. 
                |         iTargetTaskName
                |             The transformation that needs to be applied on the targets (w.r.t
                |             the robot). 
                |         oClonedRobotTask
                |             The resultant cloned robot task. 
                | 
                |     Returns:
                |         An HRESULT.
                |         Legal values:
                | 
                |         S_OK
                |             The Robot Task cloning was successful.
                |         E_FAIL
                |             The Robot Task cloning was unsuccessful.
                | 
                |         Example:
                |             The following example show how to clone a task.
                | 
                |                Dim objRobotTaskCloneFactory As
                |                RobotTaskCloneFactory
                |                Dim iSrcTask, oClonedTask 
                |                Dim iTargetRobot 
                |                Dim iTrans(0 to 11) 
                |                ..
                |                objRobotTaskCloneFactory.CloneTaskInAnotherRobot oSrcTask,
                |                iTargetRobot, "Task.1", oTrans, oClonedTask

        :param RobotTask i_src_robot_task:
        :param AnyObject i_dest_robot:
        :param str i_target_task_name:
        :param tuple i_trans_on_target:
        :param RobotTask o_cloned_robot_task:
        :rtype: None
        """
        return self.robot_task_clone_factory.CloneTaskInAnotherRobot(
            i_src_robot_task.com_object,
            i_dest_robot.com_object,
            i_target_task_name,
            i_trans_on_target,
            o_cloned_robot_task.com_object
        )
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'clone_task_in_another_robot'
        # # vba_code = """
        # # Public Function clone_task_in_another_robot(robot_task_clone_factory)
        # #     Dim iSrcRobotTask (2)
        # #     robot_task_clone_factory.CloneTaskInAnotherRobot iSrcRobotTask
        # #     clone_task_in_another_robot = iSrcRobotTask
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def clone_task_in_same_robot(
            self,
            i_src_robot_task: RobotTask,
            i_target_task_name: str,
            i_trans_on_target: tuple,
            o_cloned_robot_task: RobotTask
    ) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub CloneTaskInSameRobot(RobotTask iSrcRobotTask,
                | CATBSTR iTargetTaskName,
                | CATSafeArrayVariant iTransOnTarget,
                | RobotTask oClonedRobotTask)
                | 
                |     Clones a given Robot Task with in the same robot.
                | 
                |     Parameters:
                | 
                |         iSrcRobotTask
                |             The Robot task that needs to be cloned. 
                |         iTargetTaskName
                |             The name for the cloned robot task. 
                |         oClonedRobotTask
                |             The resultant cloned robot task. 
                |         iTargetTaskName
                |             The transformation that needs to be applied on the targets (w.r.t
                |             the robot). 
                | 
                |     Returns:
                |         An HRESULT.
                |         Legal values:
                | 
                |         S_OK
                |             The Robot Task cloning was successful.
                |         E_FAIL
                |             The Robot Task cloning was unsuccessful.
                | 
                |         Example:
                |             The following example show how to clone a task.
                | 
                |                Dim objRobotTaskCloneFactory As
                |                RobotTaskCloneFactory
                |                Dim iSrcTask, oClonedTask 
                |                Dim iTrans(0 to 11) 
                |                ..
                |                objRobotTaskCloneFactory.CloneTaskInSameRobot iSrcTask,
                |                "Task.1", iTrans, oClonedTask

        :param RobotTask i_src_robot_task:
        :param str i_target_task_name:
        :param tuple i_trans_on_target:
        :param RobotTask o_cloned_robot_task:
        :rtype: None
        """
        return self.robot_task_clone_factory.CloneTaskInSameRobot(
            i_src_robot_task.com_object,
            i_target_task_name,
            i_trans_on_target,
            o_cloned_robot_task.com_object
        )
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'clone_task_in_same_robot'
        # # vba_code = """
        # # Public Function clone_task_in_same_robot(robot_task_clone_factory)
        # #     Dim iSrcRobotTask (2)
        # #     robot_task_clone_factory.CloneTaskInSameRobot iSrcRobotTask
        # #     clone_task_in_same_robot = iSrcRobotTask
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def __repr__(self):
        return f'RobotTaskCloneFactory(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from pycatia.cat_str_functional_interfaces.sfm_manager import SFMManager
from pycatia.cat_str_functional_interfaces.sfm_member import SFMMember
from pycatia.cat_str_functional_interfaces.sfm_member2_points import SFMMember2Points
from pycatia.cat_str_functional_interfaces.sfm_member_curve import SFMMemberCurve
from pycatia.cat_str_functional_interfaces.sfm_member_plane2_curves import SFMMemberPlane2Curves
from pycatia.cat_str_functional_interfaces.sfm_member_point_length import SFMMemberPointLength
from pycatia.cat_str_functional_interfaces.sfm_member_point_up_to_limit import SFMMemberPointUpToLimit
from pycatia.cat_str_functional_interfaces.sfm_member_surf_surf import SFMMemberSurfSurf
from pycatia.cat_str_functional_interfaces.sfm_stiffener import SFMStiffener
from pycatia.cat_str_functional_interfaces.sfm_stiffener_on_free_edge import SFMStiffenerOnFreeEdge
from pycatia.cat_str_functional_interfaces.sfm_super_plate import SFMSuperPlate
from pycatia.in_interfaces.reference import Reference
from pycatia.mec_mod_interfaces.factory import Factory


class SFMFactory(Factory):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     MecModInterfaces.Factory
                |                         SfmFactory
                | 
                | Interface to create Structure Functional Modeler Objects.
                | Role: To create the structure object such as SuperPlate, Stiffener,
                | StiffenerOnFreeEdge, Beam, Opening and Connection.
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.sfm_factory = com_object

    def add_adv_super_plate(self, i_category: str, i_support: Reference, i_destination: Reference) -> SFMSuperPlate:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func AddAdvSuperPlate(CATBSTR iCategory,
                | Reference iSupport,
                | Reference iDestination) As SfmSuperPlate
                | 
                |     Creates an advanced SuperPlate.
                |     Role: Allows creating an advanced SuperPlate, ie with the concave limit
                |     mode.
                | 
                |     Parameters:
                | 
                |         iCategory
                |             [in] Category. 
                |         iSupport
                |             [in] Support surface. 
                |         iDestination
                |             [in] SuperPlate's destination. 
                |         oSuperPlate
                |             [out] SuperPlate. 
                | 
                |     Returns:
                |         S_OK if everything ran ok. 
                |     See also:
                |         SfmSuperPlate

        :param str i_category:
        :param Reference i_support:
        :param Reference i_destination:
        :rtype: SFMSuperPlate
        """
        return SFMSuperPlate(
            self.sfm_factory.AddAdvSuperPlate(
                i_category,
                i_support.com_object,
                i_destination.com_object
            )
        )

    def add_member_beams_and_plane(
            self, i_category: str,
            i_section_name: str,
            i_member1: SFMMember,
            i_member2: SFMMember,
            i_ref_plane: Reference,
            i_destination: Reference
    ) -> SFMMemberPlane2Curves:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func AddMemberBeamsAndPlane(CATBSTR iCategory,
                | CATBSTR iSectionName,
                | SfmMember iMember1,
                | SfmMember iMember2,
                | Reference iRefPlane,
                | Reference iDestination) As SfmMemberPlane2Curves
                | 
                |     Creates a Member between two existing members using Plane.
                | 
                |     Parameters:
                | 
                |         iCategory
                |             [in] Member's category. 
                |         iSectionName
                |             [in] Name of the section. 
                |         iMember1
                |             [in] First Member. 
                |         iMember1
                |             [in] Second Member. 
                |         iRefPlane
                |             [in] Plane 
                |         iDestination
                |             [in] Member's destination. 
                |         oMember
                |             [out] Member. 
                | 
                |     Returns:
                |         S_OK if everything ran ok. 
                |     See also:
                |         CATIASfmMemberPlane2Members

        :param str i_category:
        :param str i_section_name:
        :param SFMMember i_member1:
        :param SFMMember i_member2:
        :param Reference i_ref_plane:
        :param Reference i_destination:
        :rtype: SFMMemberPlane2Curves
        """
        return SFMMemberPlane2Curves(
            self.sfm_factory.AddMemberBeamsAndPlane(
                i_category,
                i_section_name,
                i_member1.com_object,
                i_member2.com_object,
                i_ref_plane.com_object,
                i_destination.com_object
            )
        )

    def add_member_crv(
            self,
            i_category: str,
            i_section_name: str,
            i_curve: Reference,
            i_destination: Reference
    ) -> SFMMemberCurve:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func AddMemberCrv(CATBSTR iCategory,
                | CATBSTR iSectionName,
                | Reference iCurve,
                | Reference iDestination) As SfmMemberCurve
                | 
                |     Creates a SuperMember on a curve.
                |     Role: Allows creating a member on a curve.
                | 
                |     Parameters:
                | 
                |         iCategory
                |             [in] Member's category. 
                |         iSectionName
                |             [in] Name of the section. 
                |         iCurve
                |             [in] Curve.
                |             Sub-element(s) supported (see 
                | 
                |         Boundary object): 
                |     iDestination
                |         [in] Member's destination. 
                |     oMember
                |         [out] Member. 
                |     Returns:
                |         S_OK if everything ran ok. 
                |     See also:
                |         SfmMemberCurve

        :param str i_category:
        :param str i_section_name:
        :param Reference i_curve:
        :param Reference i_destination:
        :rtype: SFMMemberCurve
        """
        return SFMMemberCurve(
            self.sfm_factory.AddMemberCrv(
                i_category,
                i_section_name,
                i_curve.com_object,
                i_destination.com_object
            )
        )

    def add_member_crv_on_ref(
            self,
            i_category: str,
            i_section_name: str,
            i_curve: Reference,
            i_reference: Reference,
            i_destination: Reference
    ) -> SFMMemberCurve:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func AddMemberCrvOnRef(CATBSTR iCategory,
                | CATBSTR iSectionName,
                | Reference iCurve,
                | Reference iReference,
                | Reference iDestination) As SfmMemberCurve
                | 
                |     Creates a SuperMember on a curve and on a reference
                |     surface.
                |     Role: Allows creating a member on a curve, perpendicular to a reference
                |     surface.
                | 
                |     Parameters:
                | 
                |         iCategory
                |             [in] Member's category. 
                |         iSectionName
                |             [in] Name of the section. 
                |         iCurve
                |             [in] Curve.
                |             Sub-element(s) supported (see 
                | 
                |         Boundary object): 
                |     iReference
                |         [in] Surface on which the curve must lay down. 
                |     iDestination
                |         [in] Member's destination. 
                |     oMember
                |         [out] Member. 
                |     Returns:
                |         S_OK if everything ran ok. 
                |     See also:
                |         SfmMemberCurve

        :param str i_category:
        :param str i_section_name:
        :param Reference i_curve:
        :param Reference i_reference:
        :param Reference i_destination:
        :rtype: SFMMemberCurve
        """
        return SFMMemberCurve(
            self.sfm_factory.AddMemberCrvOnRef(
                i_category,
                i_section_name,
                i_curve.com_object,
                i_reference.com_object,
                i_destination.com_object
            )
        )

    def add_member_pt_length(
            self,
            i_category: str,
            i_section_name: str,
            i_point: Reference,
            i_direction: Reference,
            i_length: float,
            i_destination: Reference
    ) -> SFMMemberPointLength:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func AddMemberPtLength(CATBSTR iCategory,
                | CATBSTR iSectionName,
                | Reference iPoint,
                | Reference iDirection,
                | double iLength,
                | Reference iDestination) As SfmMemberPointLength
                | 
                |     Creates a straight SuperMember defined by a point, a direction and a
                |     length.
                |     Role: Allows creating a member defined by a point, a direction and a
                |     length.
                | 
                |     Parameters:
                | 
                |         iCategory
                |             [in] Member's category. 
                |         iSectionName
                |             [in] Name of the section. 
                |         iPoint
                |             [in] Point. 
                |         iDirection
                |             [in] Direction element: it can be a line or a plane.
                |             
                |         iLength
                |             [in] Length. 
                |         iDestination
                |             [in] Member's destination. 
                |         oMember
                |             [out] Member. 
                | 
                |     Returns:
                |         S_OK if everything ran ok. 
                |     See also:
                |         SfmMemberPointLength

        :param str i_category:
        :param str i_section_name:
        :param Reference i_point:
        :param Reference i_direction:
        :param float i_length:
        :param Reference i_destination:
        :rtype: SFMMemberPointLength
        """
        return SFMMemberPointLength(
            self.sfm_factory.AddMemberPtLength(
                i_category,
                i_section_name,
                i_point.com_object,
                i_direction.com_object,
                i_length,
                i_destination.com_object
            )
        )

    def add_member_pt_on_crv_pt(
            self,
            i_category: str,
            i_section_name: str,
            i_curve1: Reference,
            i_ratio_mode1: bool,
            i_offset1: float,
            i_point2: Reference,
            i_destination: Reference
    ) -> SFMMember2Points:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func AddMemberPtOnCrvPt(CATBSTR iCategory,
                | CATBSTR iSectionName,
                | Reference iCurve1,
                | boolean iRatioMode1,
                | double iOffset1,
                | Reference iPoint2,
                | Reference iDestination) As SfmMember2Points
                | 
                |     Creates a straight SuperMember between a point on curve and a
                |     point.
                |     Role: Allows creating a member between a point on curve and a
                |     point.
                | 
                |     Parameters:
                | 
                |         iCategory
                |             [in] Member's category. 
                |         iSectionName
                |             [in] Name of the section. 
                |         iCurve1
                |             [in] Curve. 
                |         iRatioMode1
                |             [in] If true then Ratio, if false then Length mode for the point on
                |             curve. 
                |         iOffset1
                |             [in] Offset. 
                |         iPoint2
                |             [in] Point. 
                |         iDestination
                |             [in] Member's destination. 
                |         oMember
                |             [out] Member. 
                | 
                |     Returns:
                |         S_OK if everything ran ok. 
                |     See also:
                |         SfmMember2Points

        :param str i_category:
        :param str i_section_name:
        :param Reference i_curve1:
        :param bool i_ratio_mode1:
        :param float i_offset1:
        :param Reference i_point2:
        :param Reference i_destination:
        :rtype: SFMMember2Points
        """
        return SFMMember2Points(
            self.sfm_factory.AddMemberPtOnCrvPt(
                i_category,
                i_section_name,
                i_curve1.com_object,
                i_ratio_mode1,
                i_offset1,
                i_point2.com_object,
                i_destination.com_object
            )
        )

    def add_member_pt_on_crv_pt_on_crv(
            self, i_category: str,
            i_section_name: str,
            i_curve1: Reference,
            i_ratio_mode1: bool,
            i_offset1: float,
            i_curve2: Reference,
            i_ratio_mode2: bool,
            i_offset2: float,
            i_destination: Reference
    ) -> SFMMember2Points:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func AddMemberPtOnCrvPtOnCrv(CATBSTR iCategory,
                | CATBSTR iSectionName,
                | Reference iCurve1,
                | boolean iRatioMode1,
                | double iOffset1,
                | Reference iCurve2,
                | boolean iRatioMode2,
                | double iOffset2,
                | Reference iDestination) As SfmMember2Points
                | 
                |     Creates a straight SuperMember between two points on
                |     curve.
                |     Role: Allows creating a member between two points on
                |     curve.
                | 
                |     Parameters:
                | 
                |         iCategory
                |             [in] Member's category. 
                |         iSectionName
                |             [in] Name of the section. 
                |         iCurve1
                |             [in] Curve. 
                |         iRatioMode1
                |             [in] If true then Ratio, if false then Length mode for the point on
                |             curve. 
                |         iOffset1
                |             [in] Offset. 
                |         iCurve2
                |             [in] Curve. 
                |         iRatioMode2
                |             [in] If true then Ratio, if false then Length mode for the point on
                |             curve. 
                |         iOffset2
                |             [in] Offset. 
                |         iDestination
                |             [in] Member's destination. 
                |         oMember
                |             [out] Member. 
                | 
                |     Returns:
                |         S_OK if everything ran ok. 
                |     See also:
                |         SfmMember2Points

        :param str i_category:
        :param str i_section_name:
        :param Reference i_curve1:
        :param bool i_ratio_mode1:
        :param float i_offset1:
        :param Reference i_curve2:
        :param bool i_ratio_mode2:
        :param float i_offset2:
        :param Reference i_destination:
        :rtype: SFMMember2Points
        """
        return SFMMember2Points(
            self.sfm_factory.AddMemberPtOnCrvPtOnCrv(
                i_category,
                i_section_name,
                i_curve1.com_object,
                i_ratio_mode1,
                i_offset1,
                i_curve2.com_object,
                i_ratio_mode2,
                i_offset2,
                i_destination.com_object
            )
        )

    def add_member_pt_pt(
            self,
            i_category: str,
            i_section_name: str,
            i_point1: Reference,
            i_point2: Reference,
            i_destination: Reference
    ) -> SFMMember2Points:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func AddMemberPtPt(CATBSTR iCategory,
                | CATBSTR iSectionName,
                | Reference iPoint1,
                | Reference iPoint2,
                | Reference iDestination) As SfmMember2Points
                | 
                |     Creates a straight SuperMember between two points.
                |     Role: Allows creating a member between two points.
                | 
                |     Parameters:
                | 
                |         iCategory
                |             [in] Member's category. 
                |         iSectionName
                |             [in] Name of the section. 
                |         iPoint1
                |             [in] Point. 
                |         iPoint2
                |             [in] Point. 
                |         iDestination
                |             [in] Member's destination. 
                |         oMember
                |             [out] Member. 
                | 
                |     Returns:
                |         S_OK if everything ran ok. 
                |     See also:
                |         SfmMember2Points

        :param str i_category:
        :param str i_section_name:
        :param Reference i_point1:
        :param Reference i_point2:
        :param Reference i_destination:
        :rtype: SFMMember2Points
        """
        return SFMMember2Points(
            self.sfm_factory.AddMemberPtPt(
                i_category,
                i_section_name,
                i_point1.com_object,
                i_point2.com_object,
                i_destination.com_object
            )
        )

    def add_member_pt_pt_on_crv(
            self,
            i_category: str,
            i_section_name: str,
            i_point1: Reference,
            i_curve2: Reference,
            i_ratio_mode2: bool,
            i_offset2: float,
            i_destination: Reference
    ) -> SFMMember2Points:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func AddMemberPtPtOnCrv(CATBSTR iCategory,
                | CATBSTR iSectionName,
                | Reference iPoint1,
                | Reference iCurve2,
                | boolean iRatioMode2,
                | double iOffset2,
                | Reference iDestination) As SfmMember2Points
                | 
                |     Creates a straight SuperMember between a point and a point on
                |     curve.
                |     Role: Allows creating a member between a point and a point on
                |     curve.
                | 
                |     Parameters:
                | 
                |         iCategory
                |             [in] Member's category. 
                |         iSectionName
                |             [in] Name of the section. 
                |         iPoint1
                |             [in] Point. 
                |         iCurve2
                |             [in] Curve. 
                |         iRatioMode2
                |             [in] If true then Ratio, if false then Length mode for the point on
                |             curve. 
                |         iOffset2
                |             [in] Offset. 
                |         iDestination
                |             [in] Member's destination. 
                |         oMember
                |             [out] Member. 
                | 
                |     Returns:
                |         S_OK if everything ran ok. 
                |     See also:
                |         SfmMember2Points

        :param str i_category:
        :param str i_section_name:
        :param Reference i_point1:
        :param Reference i_curve2:
        :param bool i_ratio_mode2:
        :param float i_offset2:
        :param Reference i_destination:
        :rtype: SFMMember2Points
        """
        return SFMMember2Points(
            self.sfm_factory.AddMemberPtPtOnCrv(
                i_category,
                i_section_name,
                i_point1.com_object,
                i_curve2.com_object,
                i_ratio_mode2,
                i_offset2,
                i_destination.com_object
            )
        )

    def add_member_pt_up_to_limit(
            self,
            i_category: str,
            i_section_name: str,
            i_point: Reference,
            i_direction: Reference,
            i_limit: Reference,
            i_destination: Reference
    ) -> SFMMemberPointUpToLimit:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func AddMemberPtUpToLimit(CATBSTR iCategory,
                | CATBSTR iSectionName,
                | Reference iPoint,
                | Reference iDirection,
                | Reference iLimit,
                | Reference iDestination) As SfmMemberPointUpToLimit
                | 
                |     Creates a straight member defined by a point, a direction and a
                |     limit.
                |     Role: Allows creating a member defined by a point, a direction and a
                |     limit.
                | 
                |     Parameters:
                | 
                |         iCategory
                |             [in] Member's category. 
                |         iSectionName
                |             [in] Name of the section. 
                |         iPoint
                |             [in] Point. 
                |         iDirection
                |             [in] Direction element: it can be a line or a plane.
                |             
                |         iLimit
                |             [in] Limit: it can be any kind of geometric element.
                |             
                |         iDestination
                |             [in] Member's destination. 
                |         oMember
                |             [out] Member. 
                | 
                |     Returns:
                |         S_OK if everything ran ok. 
                |     See also:
                |         SfmMemberPointUpToLimit

        :param str i_category:
        :param str i_section_name:
        :param Reference i_point:
        :param Reference i_direction:
        :param Reference i_limit:
        :param Reference i_destination:
        :rtype: SFMMemberPointUpToLimit
        """
        return SFMMemberPointUpToLimit(
            self.sfm_factory.AddMemberPtUpToLimit(
                i_category,
                i_section_name,
                i_point.com_object,
                i_direction.com_object,
                i_limit.com_object,
                i_destination.com_object
            )
        )

    def add_member_surf_surf(
            self,
            i_category: str,
            i_section_name: str,
            i_surface1: Reference,
            i_surface2: Reference,
            i_destination: Reference
    ) -> SFMMemberSurfSurf:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func AddMemberSurfSurf(CATBSTR iCategory,
                | CATBSTR iSectionName,
                | Reference iSurface1,
                | Reference iSurface2,
                | Reference iDestination) As SfmMemberSurfSurf
                | 
                |     Creates a SuperMember by the intersection of two surfaces.
                |     Role: Allows creating a member by the intersection of two
                |     surfaces.
                | 
                |     Parameters:
                | 
                |         iCategory
                |             [in] Member's category. 
                |         iSectionName
                |             [in] Name of the section. 
                |         iSurface1
                |             [in] First surface. 
                |         iSurface2
                |             [in] Second surface. 
                |         iDestination
                |             [in] Member's destination. 
                |         oMember
                |             [out] Member. 
                | 
                |     Returns:
                |         S_OK if everything ran ok. 
                |     See also:
                |         SfmMemberSurfSurf

        :param str i_category:
        :param str i_section_name:
        :param Reference i_surface1:
        :param Reference i_surface2:
        :param Reference i_destination:
        :rtype: SFMMemberSurfSurf
        """
        return SFMMemberSurfSurf(
            self.sfm_factory.AddMemberSurfSurf(
                i_category,
                i_section_name,
                i_surface1.com_object,
                i_surface2.com_object,
                i_destination.com_object
            )
        )

    def add_super_plate(self, i_category: str, i_support: Reference, i_destination: Reference) -> SFMSuperPlate:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func AddSuperPlate(CATBSTR iCategory,
                | Reference iSupport,
                | Reference iDestination) As SfmSuperPlate
                | 
                |     Creates a SuperPlate.
                |     Role: Allows creating a SuperPlate in the regular limit mode (split
                |     mode).
                | 
                |     Parameters:
                | 
                |         iCategory
                |             [in] Category. 
                |         iSupport
                |             [in] Support surface. 
                |         iDestination
                |             [in] SuperPlate's destination. 
                |         oSuperPlate
                |             [out] SuperPlate. 
                | 
                |     Returns:
                |         S_OK if everything ran ok. 
                |     See also:
                |         SfmSuperPlate

        :param str i_category:
        :param Reference i_support:
        :param Reference i_destination:
        :rtype: SFMSuperPlate
        """
        return SFMSuperPlate(self.sfm_factory.AddSuperPlate(i_category, i_support.com_object, i_destination.com_object))

    def add_super_stiffener(self, i_category: str, i_section_name: str, i_super_plate: SFMSuperPlate,
                            i_web_support: Reference) -> SFMStiffener:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func AddSuperStiffener(CATBSTR iCategory,
                | CATBSTR iSectionName,
                | SfmSuperPlate iSuperPlate,
                | Reference iWebSupport) As SfmStiffener
                | 
                |     Creates a SuperStiffener.
                |     Role: Allows creating a SuperStiffener in the normal to plate mode with the
                |     WebSideLeft anchor point.
                | 
                |     Parameters:
                | 
                |         iCategory
                |             [in] Category. 
                |         iSectionName
                |             [in] Section name. 
                |         iSuperPlate
                |             [in] SuperPlate to be stiffened. 
                |         oSuperStiffener
                |             [out] SuperStiffener. 
                | 
                |     Returns:
                |         S_OK if everything ran ok. 
                |     See also:
                |         SfmStiffener

        :param str i_category:
        :param str i_section_name:
        :param SFMSuperPlate i_super_plate:
        :param Reference i_web_support:
        :rtype: SFMStiffener
        """
        return SFMStiffener(
            self.sfm_factory.AddSuperStiffener(
                i_category,
                i_section_name,
                i_super_plate.com_object,
                i_web_support.com_object
            )
        )

    def add_super_stiffener_on_free_edge(
            self,
            i_category: str,
            i_section_name: str,
            i_super_plate: SFMSuperPlate,
            i_free_edge: Reference
    ) -> SFMStiffenerOnFreeEdge:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func AddSuperStiffenerOnFreeEdge(CATBSTR iCategory,
                | CATBSTR iSectionName,
                | SfmSuperPlate iSuperPlate,
                | Reference iFreeEdge) As SfmStiffenerOnFreeEdge
                | 
                |     Creates a SuperStiffener on Free Edge.
                |     Role: Allows creating a SuperStiffener on Free Edge in the normal to plate
                |     mode with the WebSideLeft anchor point.
                | 
                |     Parameters:
                | 
                |         iCategory
                |             [in] Category. 
                |         iSectionName
                |             [in] Section name. 
                |         iSuperPlate
                |             [in] SuperPlate to be stiffened. 
                |         iFreeEdge
                |             [in] Support to be used for creating SFE. 
                |         oSuperStiffenerOnFreeEdge
                |             [out] SuperStiffener on free edge. 
                | 
                |     Returns:
                |         S_OK if everything ran ok. 
                |     See also:
                |         SfmStiffenerOnFreeEdge

        :param str i_category:
        :param str i_section_name:
        :param SFMSuperPlate i_super_plate:
        :param Reference i_free_edge:
        :rtype: SFMStiffenerOnFreeEdge
        """
        return SFMStiffenerOnFreeEdge(
            self.sfm_factory.AddSuperStiffenerOnFreeEdge(
                i_category,
                i_section_name,
                i_super_plate.com_object,
                i_free_edge.com_object
            )
        )

    def get_manager(self) -> SFMManager:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func GetManager() As SfmManager
                | 
                |     Retrieved the services manager.
                |     Role: Allows getting the services manager.
                | 
                |     Returns:
                |         S_OK if everything ran ok. 
                |     See also:
                |         SfmManager

        :rtype: SFMManager
        """
        return SFMManager(self.sfm_factory.GetManager())

    def __repr__(self):
        return f'SFMFactory(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from pycatia.dmaps_interfaces.activities import Activities
from pycatia.in_interfaces.document import Document
from pycatia.ppr_interfaces.ppr_products import PPRProducts


class PPRDocument(Document):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     InfInterfaces.Document
                |                         PPRDocument
                | 
                | This interface is used to access Processes, Products and
                | Resources.
                | Due to the inherent design restrictions, PPRDocument and another interface ProcessDocument need to
                | be used with care, since the ActiveDocument could be either ProcessDocument or PPRDocument. With that
                | in mind, a good practice of writing more robust VB script is to first tell the real type of document
                | (interface).
                | E.g. The following sentence
                | Dim MyDoc As PPRDocument
                | Set MyDoc = DELMIA.ActiveDocument.PPRDocument
                | could be replaced by the following:
                | Dim MyDoc As PPRDocument
                | Set MyProcDoc = DELMIA.ActiveDocument
                | if (TypeName(MyProcDoc)="ProcessDocument")
                | then MyDoc = MyProcDoc.PPRDocument
                | else MyDoc = DELMIA.ActiveDocument
                | end if
    
    """

    def __init__(self, com_object):
        super().__init__(com_object)
        self.ppr_document = com_object

    @property
    def processes(self) -> Activities:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property Processes() As Activities (Read Only)
                | 
                |     Retrieves the list of Processes as Activities contained in the current
                |     document.
                | 
                |     Returns:
                |         The list of Activities contained in the current document.

        :rtype: Activities
        """

        return Activities(self.ppr_document.Processes)

    @property
    def products(self) -> PPRProducts:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property Products() As PPRProducts (Read Only)
                | 
                |     Retrieves the list of Products contained in the current
                |     document.
                | 
                |     Returns:
                |         The list of Products contained in the current document.

        :rtype: PPRProducts
        """

        return PPRProducts(self.ppr_document.Products)

    @property
    def resources(self) -> PPRProducts:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)
                | o Property Resources() As PPRProducts (Read Only)
                | 
                |     Retrieves the list of Resources contained in the current
                |     document.
                | 
                |     Returns:
                |         The list of Resources contained in the current document.

        :rtype: PPRProducts
        """

        return PPRProducts(self.ppr_document.Resources)

    def __repr__(self):
        return f'PprDocument(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-09-25 14:34:21.593357

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.
        
"""
from pycatia.funct_system_interfaces.funct_actions_group import FunctActionsGroup
from pycatia.system_interfaces.collection import Collection
from pycatia.types.general import cat_variant


class FunctActionsGroups(Collection):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.Collection
                |                     FunctActionsGroups
                | 
                | The interface to access the groups of functional actions in a
                | system.
    
    """

    def __init__(self, com_object, child_object=FunctActionsGroup):
        super().__init__(com_object, child_object=FunctActionsGroup)
        self.funct_actions_groups = com_object
        self.child_object = child_object

    def create(self, i_name: str, i_input_x: float, i_input_y: float, i_output_x: float,
               i_output_y: float) -> FunctActionsGroup:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func Create(CATBSTR iName,
                | double iInputX,
                | double iInputY,
                | double iOutputX,
                | double iOutputY) As FunctActionsGroup
                | 
                |     Create a Group of Actions.

        :param str i_name:
        :param float i_input_x:
        :param float i_input_y:
        :param float i_output_x:
        :param float i_output_y:
        :rtype: FunctActionsGroup
        """
        return FunctActionsGroup(self.funct_actions_groups.Create(i_name, i_input_x, i_input_y, i_output_x, i_output_y))

    def delete(self, i_act_grp: FunctActionsGroup) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Sub Delete(FunctActionsGroup iActGrp)
                | 
                |     Delete a Group of Actions.

        :param FunctActionsGroup i_act_grp:
        :rtype: None
        """
        return self.funct_actions_groups.Delete(i_act_grp.com_object)
        # # # # Autogenerated comment: 
        # # some methods require a system service call as the methods expects a vb array object
        # # passed to it and there is no way to do this directly with python. In those cases the following code
        # # should be uncommented and edited accordingly. Otherwise completely remove all this.
        # # vba_function_name = 'delete'
        # # vba_code = """
        # # Public Function delete(funct_actions_groups)
        # #     Dim iActGrp (2)
        # #     funct_actions_groups.Delete iActGrp
        # #     delete = iActGrp
        # # End Function
        # # """

        # # system_service = SystemService(self.application.SystemService)
        # # return system_service.evaluate(vba_code, 0, vba_function_name, [self.com_object])

    def elem(self, i_index: cat_variant) -> FunctActionsGroup:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-09-25 14:34:21.593357))
                | o Func Elem(CATVariant iIndex) As FunctActionsGroup
                | 
                |     Returns an actions' group using its index or its name from the actions'
                |     groups collection.
                | 
                |     Parameters:
                | 
                |         iIndex
                |             The index or the name of the actions' group to retrieve from the
                |             collection of actions' groups. As a numerics, this index is the rank of the
                |             actions' group in the collection. The index of the first actions' group in the
                |             collection is 1, and the index of the last actions' group is Count. As a
                |             string, it is the name you assigned to the actions' group using the
                |             
                | 
                |         AnyObject.Name property. 
                |     Returns:
                |         The retrieved action 
                |     Example:
                |         This example retrieves in AG1 the fifth actions' group in the
                |         collection and in AG2 the actions' group named
                |         Transmission.
                | 
                |          Dim AG1 As FunctActionsGroup
                |          Set AG1 = ActionsGroups.Elem(5)
                |          Dim AG2 As FunctActionsGroup
                |          Set AG2 = ActionsGroups.Elem("Transmission")

        :param cat_variant i_index:
        :rtype: FunctActionsGroup
        """
        return FunctActionsGroup(self.funct_actions_groups.Elem(i_index))

    def __repr__(self):
        return f'FunctActionsGroups(name="{self.name}")'



#! usr/bin/python3.9
"""
    Module initially auto generated using V5Automation files from CATIA V5 R28 on 2020-06-11 12:40:47.360445

    .. warning::
        The notes denoted "CAA V5 Visual Basic Help" are to be used as reference only.
        They are there as a guide as to how the visual basic / catscript functions work
        and thus help debugging in pycatia.

"""

from pathlib import Path

from pycatia.exception_handling.exceptions import CATIAApplicationException
from pycatia.hybrid_shape_interfaces.hybrid_shape_factory import HybridShapeFactory
from pycatia.in_interfaces.reference import Reference
from pycatia.knowledge_interfaces.parameters import Parameters
from pycatia.knowledge_interfaces.relations import Relations
from pycatia.mec_mod_interfaces.axis_systems import AxisSystems
from pycatia.mec_mod_interfaces.bodies import Bodies
from pycatia.mec_mod_interfaces.body import Body
from pycatia.mec_mod_interfaces.constraints import Constraints
from pycatia.mec_mod_interfaces.factory import Factory
from pycatia.mec_mod_interfaces.geometric_elements import GeometricElements
from pycatia.mec_mod_interfaces.hybrid_bodies import HybridBodies
from pycatia.mec_mod_interfaces.ordered_geometrical_sets import OrderedGeometricalSets
from pycatia.mec_mod_interfaces.origin_elements import OriginElements
from pycatia.part_interfaces.shape_factory import ShapeFactory
from pycatia.system_interfaces.any_object import AnyObject
from pycatia.product_structure_interfaces.analyze import Analyze
from pycatia.system_interfaces.collection import Collection
from pycatia.cat_tps_interfaces.annotation_sets import AnnotationSets


class Part(AnyObject):
    """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)

                | System.IUnknown
                |     System.IDispatch
                |         System.CATBaseUnknown
                |             System.CATBaseDispatch
                |                 System.AnyObject
                |                     Part
                |
                | The root level object inside a PartDocument object.
                | Role: It aggregates all the objects making up the part
                | document.
                | It provides many factories and collections. The collections list only the
                | direct children of the part. Selection.Search allows to get all objects of one
                | type.
                |
                | See also:
                |     PartDocument

    """

    def __init__(self, com_part_object):
        super().__init__(com_part_object)
        self.part = com_part_object
        self.com_object = com_part_object

    @property
    def analyze(self) -> Analyze:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Analyze() As Analyze (Read Only)
                |
                |     Returns the Analyze object associated to the current
                |     product.
                |
                |     Example:
                |
                |           This example retrieves in EngineAnalysis the Analyze object
                |           of
                |          the Engine product.
                |
                |
                |          Dim EngineAnalysis As Analyze
                |          Set EngineAnalysis = Engine.Analyze

        :rtype: Analyze
        """

        return Analyze(self.part.Analyze)

    @property
    def annotation_sets(self) -> Collection:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property AnnotationSets() As Collection (Read Only)
                | 
                |     Returns the collection object containing the annotation sets. All the
                |     annotation sets that are aggregated in the part might be accessed thru that
                |     collection.
                |
                |     Example:
                |         The following example returns in annotationSets the annotation sets of
                |         the partRoot part from the partDoc part document:
                |
                |          Set partRoot = partDoc.Part
                |          Dim annotationSets As AnnotationSets
                |          Set annotationSets = partRoot.AnnotationSets

        :return: Collection
        """

        return AnnotationSets(self.part.AnnotationSets)

    @property
    def axis_systems(self) -> AxisSystems:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property AxisSystems() As AxisSystems (Read Only)
                |
                |     Returns the collection object containing the coordinate systems. All the
                |     coordinate systems that are aggregated in the part might be accessed thru that
                |     collection.
                |
                |     Example:
                |         The following example returns in axisSystems the coordinate systems of
                |         the partRoot part from the partDoc part document:
                | 
                |          Set partRoot = partDoc.Part
                |          Dim axisSystems As AxisSystems
                |          Set axisSystems = partRoot.AxisSystems

        :return: AxisSystems

        This will only return axis systems under the Axis Systems node. Axis systems within a Geometrical Set will not
        be returned.
        """

        return AxisSystems(self.part.AxisSystems)

    @property
    def bodies(self) -> Bodies:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Bodies() As Bodies (Read Only)
                | 
                |     Returns the collection object containing the bodies that are direct
                |     children of the part.
                |     It does not return all the bodies of the part, particularly the bodies in a
                |     boolean operation.
                |
                |     Example:
                |         The following example returns in bodiesColl the collection of the
                |         bodies of the partRoot part from the partDoc part
                |         document:
                |
                |          Set partRoot = partDoc.Part
                |          Set bodiesColl = partRoot.Bodies

        :rtype: Bodies
        """

        return Bodies(self.part.Bodies)

    @property
    def constraints(self) -> Constraints:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Constraints() As Constraints (Read Only)
                | 
                |     Returns the collection object containing the part constraints. Only 3D
                |     constraints are concerned here, 2D constraints are managed in
                |     sketches.
                |
                |     Example:
                |         The following example returns in csts the constraints of the partRoot
                |         part from the partDoc part document:
                |
                |          Set partRoot = partDoc.Part
                |          Set csts = partRoot.Constraints

        :rtype: Constraints
        """

        return Constraints(self.part.Constraints)

    @property
    def density(self) -> float:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Density() As double (Read Only)
                |
                |     Returns the part density.
                | 
                |     Example:
                |         The following example displays the density of the
                |         part:
                |
                |          Set partRoot = partDoc.Part
                |          MsgBox "The density is " & partRoot.Density

        :rtype: float
        """

        return self.part.Density

    @property
    def file_name(self) -> str:
        """
        :rtype: str
        """
        try:
            return self.part.ReferenceProduct.Parent.Name
        except AttributeError:
            return self.part.Name

    @property
    def full_name(self) -> str:
        """
        :rtype: str
        """
        try:
            return self.part.ReferenceProduct.Parent.FullName
        except AttributeError:
            return self.part.FullName

    @property
    def geometric_elements(self) -> GeometricElements:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property GeometricElements() As GeometricElements (Read
                | Only)
                |
                |     Returns the collection object containing the part geometrical elements.
                |     Only 3D elements are concerned here, 2D elements are managed in sketches. The
                |     origin elements are also accessible thru that collection.
                |
                |     Example:
                |         The following example returns in geomElts the 3D elements of the
                |         partRoot part from the partDoc part document:
                | 
                |          Set partRoot = partDoc.Part
                |          Set geomElts = partRoot.GeometricElements

        :rtype: GeometricElements
        """

        return GeometricElements(self.part.GeometricElements)

    @property
    def hybrid_bodies(self) -> HybridBodies:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property HybridBodies() As HybridBodies (Read Only)
                |
                |     Returns the collection object containing the hybrid bodies that are direct
                |     children of the part.
                |
                |     Example:
                |         The following example returns in hybridBodiesColl the collection of
                |         hybrid bodies of the partRoot part from the partDoc part
                |         document:
                | 
                |          Set partRoot = partDoc.Part
                |          Set hybridBodiesColl = partRoot.HybridBodies

        :rtype: HybridBodies
        """

        return HybridBodies(self.part.HybridBodies)

    @property
    def hybrid_shape_factory(self) -> HybridShapeFactory:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property HybridShapeFactory() As Factory (Read Only)
                | 
                |     Returns the part hybrid shape factory. It allows the creation of hybrid
                |     shapes in the part.
                |
                |     Example:
                |         The following example returns in hybridShapeFact the hybrid shape
                |         factory of the partRoot part from the partDoc part
                |         document:
                |
                |          Set partRoot = partDoc.Part
                |          Dim hybridShapeFact As Factory
                |          Set hybridShapeFact = partRoot.HybridShapeFactory

        :rtype: HybridShapeFactory
        """

        return HybridShapeFactory(self.part.HybridShapeFactory)

    @property
    def in_work_object(self) -> AnyObject:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property InWorkObject() As AnyObject
                |
                |     Returns or sets the in work object of the part. The in work object is the
                |     object after which a new object is added.
                |
                |     Example:
                |
                |      Set partRoot = partDoc.Part
                |      Set partRoot.InWorkObject = cylindricPad
                |      If ( partRoot.InWorkObject <> cylindricPad ) Then
                |           MsgBox "There is a big problem"
                |      End If

        :rtype: AnyObject
        """

        return AnyObject(self.part.InWorkObject)

    @in_work_object.setter
    def in_work_object(self, any_object: AnyObject):
        """
        :param AnyObject any_object:
        """

        self.part.InWorkObject = any_object.com_object

    @property
    def main_body(self) -> Body:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property MainBody() As Body
                |
                |     Returns or sets the main body of the part.
                |
                |     Example:
                |         The following example returns the main body of the part of the current
                |         document.
                |
                |          Dim mainBody As Body
                |          Set mainBody=CATIA.ActiveDocument.Part.MainBody

        :rtype: Body
        """

        return Body(self.part.MainBody)

    @main_body.setter
    def main_body(self, body: Body):
        """
        :param Body body:
        """

        self.part.MainBody = body.com_object

    @property
    def ordered_geometrical_sets(self) -> OrderedGeometricalSets:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property OrderedGeometricalSets() As OrderedGeometricalSets (Read
                | Only)
                |
                |     Returns the collection object containing the ordered geometrical sets of
                |     the part.
                |
                |     Example:
                |         The following example returns in ogsColl the collection of ordered
                |         geometrical sets of the partRoot part from the partDoc part
                |         document:
                |
                |          Set partRoot = partDoc.Part
                |          Set ogsColl = partRoot.OrderedGeometricalSets

        :rtype: OrderedGeometricalSets
        """

        return OrderedGeometricalSets(self.part.OrderedGeometricalSets)

    @property
    def origin_elements(self) -> OriginElements:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property OriginElements() As OriginElements (Read Only)
                | 
                |     Returns the object defining the part 3D reference axis
                |     system.
                |
                |     Example:
                |         The following example returns in originElts the origin of the partRoot
                |         part from the partDoc part document:
                |
                |          Set partRoot = partDoc.Part
                |          Set originElts = partRoot.OriginElements

        :rtype: OriginElements
        """

        return OriginElements(self.part.OriginElements)

    @property
    def parameters(self) -> Parameters:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Parameters() As Parameters (Read Only)
                |
                |     Returns the collection object containing the part parameters. All the
                |     parameters that are aggregated in the different objects of the part might be
                |     accessed thru that collection.
                |
                |     Example:
                |         The following example returns in params the parameters of the partRoot
                |         part from the partDoc part document:
                |
                |          Set partRoot = partDoc.Part
                |          Dim params As Parameters
                |          Set params = partRoot.Parameters

        :rtype: Parameters
        """

        return Parameters(self.part.Parameters)

    @property
    def relations(self) -> Relations:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property Relations() As Relations (Read Only)
                |
                |     Returns the collection object containing the part relations. All the
                |     relations that are used to valuate the parameters of the part might be accessed
                |     thru that collection.
                |
                |     Example:
                |         The following example returns in rels the relations of the partRoot
                |         part from the partDoc part document:
                |
                |          Set partRoot = partDoc.Part
                |          Set rels = partRoot.Relations

        :rtype: Relations
        """

        return Relations(self.part.Relations)

    @property
    def shape_factory(self) -> ShapeFactory:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property ShapeFactory() As Factory (Read Only)
                |
                |     Returns the part shape factory. It allows the creation of shapes in the
                |     part.
                |
                |     Example:
                |         The following example returns in shapeFact the shape factory of the
                |         partRoot part from the partDoc part document:
                |
                |          Set partRoot = partDoc.Part
                |          Dim shapeFact As Factory
                |          Set shapeFact = partRoot.ShapeFactory

        :rtype: ShapeFactory
        """

        return ShapeFactory(self.part.ShapeFactory)

    @property
    def sheet_metal_factory(self) -> Factory:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property SheetMetalFactory() As Factory (Read Only)
                |
                |     Returns the sheet metal factory of the part. It allows the creation of
                |     sheet metal elements in the part.
                |
                |     Example:
                |         The following example returns in sheetMetalFact the sheet metal factory
                |         of the partRoot part from the partDoc part document:
                |
                |          Set partRoot = partDoc.Part
                |          Dim sheetMetalFact As Factory
                |          Set sheetMetalFact = partRoot.SheetMetalFactory

        :return: Factory
        :return: Factory
        """

        return Factory(self.part.SheetMetalFactory)

    @property
    def sheet_metal_parameters(self) -> AnyObject:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property SheetMetalParameters() As AnyObject (Read Only)
                |
                |     Returns the sheet metal parameters of the part.
                |
                |     Example:
                |         The following example returns in sheetMetalParm the sheet metal
                |         parameters of the partRoot part from the partDoc part
                |         document:
                |
                |          Set partRoot = partDoc.Part
                |          Dim sheetMetalParm As SheetMetalParameters
                |          Set sheetMetalFact = partRoot.SheetMetalParameters

        :rtype: AnyObject
        """

        return AnyObject(self.part.SheetMetalParameters)

    @property
    def user_surfaces(self) -> Collection:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445)
                | o Property UserSurfaces() As Collection (Read Only)
                |
                |     Returns the collection object containing the user surfaces. All the user
                |     surfaces that are aggregated in the part might be accessed thru that
                |     collection.
                |
                |     Example:
                |         The following example returns in userSurfaces the user surfaces of the
                |         partRoot part from the partDoc part document:
                |
                |          Set partRoot = partDoc.Part
                |          Dim userSurfaces As UserSurfaces
                |          Set userSurfaces = partRoot.UserSurfaces

        :rtype: Collection
        """

        return Collection(self.part.UserSurfaces)

    def activate(self, i_object: AnyObject) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub Activate(AnyObject iObject)
                |
                |     Unsuppresses an object for the update process. A unsuppressed object is
                |     again taken into account for the calculation of the part.
                |
                |     Parameters:
                |
                |         iObject
                |             The object to unsuppress for the update process
                |
                |     Example:
                |         The following example unsuppresses the pad1 pad:
                |
                |          Set partRoot = partDoc.Part
                |          Set pad1 = partRoot.FindObjectByName("Pad.1")
                |          partRoot.Activate(pad1)

        :param AnyObject i_object:
        :rtype: None
        """
        return self.part.Activate(i_object.com_object)

    def create_reference_from_b_rep_name(self, i_label: str, i_object_context: AnyObject):
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func CreateReferenceFromBRepName(CATBSTR iLabel,
                | AnyObject iObjectContext) As Reference
                |
                |     Creates a reference from a GenericNaming label. This allows manipulation of
                |     B-Rep (Type Functinal and Relimited) that are not easy to
                |     access.
                |
                |     Parameters:
                |
                |         iLabel
                |             The GenericNaming identification for an object. This is a cryptic
                |             form for "the edge surrounded by the face extruded from line.12 of sketch.4 and
                |             the face...")
                |         iObjectContext
                |             The Object Context of Resolution This is the feature used for label
                |             GenericNaming resolution
                |
                |     Returns:
                |         The reference to a B-Rep sub-element such a face or an edge

        :param str i_label:
        :param AnyObject i_object_context:
        :rtype: Reference
        """
        return Reference(self.part.CreateReferenceFromBRepName(i_label, i_object_context.com_object))

    def create_reference_from_geometry(self, i_object: AnyObject) -> Reference:
        """
        .. note::
            :class: toggle

            Microsoft Visual Basic Object Browser
                | Function CreateReferenceFromGeometry(iObject As AnyObject) As Reference
                |     Member of MECMOD.Part

        :param AnyObject i_object:
        :rtype: Reference
        """

        return Reference(self.part.CreateReferenceFromGeometry(i_object.com_object))

    def create_reference_from_name(self, i_label: str):
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func CreateReferenceFromName(CATBSTR iLabel) As Reference
                |
                |     Creates a reference from a GenericNaming label. This allows manipulation of
                |     B-Rep (type Functional Only) that are not easy to access.
                |
                |     Parameters:
                |
                |         iLabel
                |             The GenericNaming identification for an object. This is a cryptic
                |             form for "the edge surrounded by the face extruded from line.12 of sketch.4 and
                |             the face...")
                |
                |     Returns:
                |         The reference to a B-Rep sub-element such a face or an edge

        :param str i_label:
        :rtype: Reference
        """
        return Reference(self.part.CreateReferenceFromName(i_label))

    def create_reference_from_object(self, i_object: AnyObject):
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func CreateReferenceFromObject(AnyObject iObject) As
                | Reference
                |
                |     Creates a reference from a operator. Use of reference allows a uniform
                |     handling of B-Rep and non B-Rep objects.
                |
                |     Parameters:
                |
                |         iObject
                |             The geometric object to be referenced. It can be a plane, a line or
                |             a point.
                |
                |     Returns:
                |         The reference to the object. This way, a direction can be either an
                |         edge of a pad or a 3D line.

        :param AnyObject i_object:
        :rtype: Reference
        """
        return Reference(self.part.CreateReferenceFromObject(i_object.com_object))

    def deactivate(self, i_object: AnyObject) -> None:
        """
        See inactivate
        :param i_object:
        :rtype: None
        """
        return self.part.Inactivate(i_object.com_object)

    def find_object_by_name(self, i_obj_name: str) -> AnyObject:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func FindObjectByName(CATBSTR iObjName) As AnyObject
                |
                |     Finds an object that is not a collection by its name. Scan in depth among
                |     all the direct and indirect children (expensive, but hard to
                |     escape).
                |
                |     Parameters:
                |
                |         iObjName
                |             The name to be searched
                |
                |     Returns:
                |         The object, if found
                |     Example:
                |         The following example tests if the object was found:
                |
                |          Set partRoot = partDoc.Part
                |          Set obj = partRoot.FindObjectByName("Wrong name")
                |          If TypeName(obj)="Nothing" Then
                |               MsgBox "Object not found"
                |          End If

        :param str i_obj_name:
        :rtype: AnyObject
        """

        if self.part.FindObjectByName(i_obj_name):
            return AnyObject(self.part.FindObjectByName(i_obj_name))
        else:
            raise CATIAApplicationException('Could not find object.')

    def get_customer_factory(self, i_factory_iid: str) -> Factory:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func GetCustomerFactory(CATBSTR iFactoryIID) As Factory
                |
                |     Returns a customer factory from a code string defined by the customer. It
                |     allows a customer to define its own factory to create its own
                |     objects.
                |
                |     Parameters:
                |
                |         iFactoryIID
                |             The code name of the factory

        :param str i_factory_iid:
        :rtype: Factory
        """
        return Factory(self.part.GetCustomerFactory(i_factory_iid))

    def inactivate(self, i_object: AnyObject) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub Inactivate(AnyObject iObject)
                |
                |     Suppresses an object from being updated. A suppressed object is not taken
                |     into account for the calculation of the part.
                |
                |     Parameters:
                |
                |         iObject
                |             The object to suppress from being updated
                |
                |     Example:
                |         The following example suppresses the pad1 pad from being
                |         updated:
                |
                |          Set partRoot = partDoc.Part
                |          Set pad1 = partRoot.FindObjectByName("Pad.1")
                |          partRoot.Inactivate(pad1)

        :param AnyObject i_object:
        :rtype: None
        """
        return self.part.Inactivate(i_object.com_object)

    def is_inactive(self, i_object: AnyObject) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func IsInactive(AnyObject iObject) As boolean
                |
                |     Indicates whether an object is deactivated. A deactivated object is not
                |     taken into account for the calculation of the part.
                |
                |     Parameters:
                |
                |         iObject
                |             The object to examine
                |
                |     Example:
                |         The following example returns in isInactive whether the pad1 pad is
                |         deactivated:
                |
                |          Set partRoot = partDoc.Part
                |          Set pad1 = partRoot.FindObjectByName("Pad.1")
                |          isInactive = partRoot.IsInactive(pad1)

        :param AnyObject i_object:
        :rtype: bool
        """
        return self.part.IsInactive(i_object.com_object)

    def is_up_to_date(self, i_object: AnyObject) -> bool:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Func IsUpToDate(AnyObject iObject) As boolean
                |
                |     Indicates whether an object needs to be updated. An object which is not
                |     up-to-date has not be calculated with the last
                |     specifications.
                |
                |     Parameters:
                |
                |         iObject
                |             The object to examine
                |
                |     Example:
                |         The following example returns in isuptodate whether the pad1 pad is
                |         up-to-date:
                |
                |          Set partRoot = partDoc.Part
                |          Set pad1 = partRoot.FindObjectByName("Pad.1")
                |          isuptodate = partRoot.IsUpToDate(pad1)

        :param AnyObject i_object:
        :rtype: bool
        """
        return self.part.IsUpToDate(i_object.com_object)

    def path(self) -> Path:
        """

        Returns the pathlib.Path() object of the document fullname.
        example e://users//psr//Parts//MyNicePart.CATPart
        >>> Part.path().name
        MyNicePart.CATPart
        >>> Part.path().parent
        e://users//psr//Parts//
        >>> Part.path().suffix
        .CATPart

        :rtype: Path()
        """
        return Path(self.full_name)

    def update(self) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub Update()
                | 
                |     Updates of the part result with respect to its specifications. Any
                |     composing specification that hasn't its result up-to-date will recompute it,
                |     thus propagating changes to the whole part.
                |
                |     Example:
                |         The following example update the part:
                |
                |          Set partRoot = partDoc.Part
                |          partRoot.Update

        :rtype: None
        """
        return self.part.Update()

    def update_object(self, i_object: AnyObject) -> None:
        """
        .. note::
            :class: toggle

            CAA V5 Visual Basic Help (2020-06-11 12:40:47.360445))
                | o Sub UpdateObject(AnyObject iObject)
                | 
                |     Updates an object with respect to its specifications. Any composing
                |     specification of the object that hasn't its result up-to-date will recompute
                |     it, thus propagating changes to the object.
                |
                |     Parameters:
                |
                |         iObject
                |             The object to be updated
                |
                |     Example:
                |         The following example updates Pad.1:
                |
                |          Set partRoot = partDoc.Part
                |          Set pad1 = partRoot.FindObjectByName("Pad.1")
                |          partRoot.UpdateObject(pad1)

        :param AnyObject i_object:
        :rtype: None
        """
        return self.part.UpdateObject(i_object.com_object)

    def __repr__(self):
        return f'Part(name="{self.name}")'
